@@ -11,7 +11,7 @@
  */
 
 import { describe, it, beforeAll, expect } from 'vitest';
-import { TestRig, validateModelOutput } from './test-helper.js';
+import { TestRig, poll, validateModelOutput } from './test-helper.js';
 import { join } from 'node:path';
 import { writeFileSync } from 'node:fs';
 
@@ -192,7 +192,7 @@ describe('simple-mcp-server', () => {
 
     // Poll for script for up to 5s
     const { accessSync, constants } = await import('node:fs');
-    const isReady = await rig.poll(
+    const isReady = await poll(
       () => {
         try {
           accessSync(testServerPath, constants.F_OK);
@@ -213,7 +213,9 @@ describe('simple-mcp-server', () => {
   it('should add two numbers', async () => {
     // Test directory is already set up in before hook
     // Just run the command - MCP server config is in settings.json
-    const output = await rig.run('add 5 and 10');
+    const output = await rig.run(
+      'Use the `add` tool to calculate 5+10 and output only the resulting number.',
+    );
 
     const foundToolCall = await rig.waitForToolCall('add');
 
