@@ -8,7 +8,7 @@
 
 import './src/gemini.js';
 import { main } from './src/gemini.js';
-import { FatalError } from '@google/gemini-cli-core';
+import { debugLogger, FatalError } from '@google/gemini-cli-core';
 
 // --- Global Entry Point ---
 main().catch((error) => {
@@ -17,14 +17,14 @@ main().catch((error) => {
     if (!process.env['NO_COLOR']) {
       errorMessage = `\x1b[31m${errorMessage}\x1b[0m`;
     }
-    console.error(errorMessage);
+    debugLogger.error(errorMessage);
     process.exit(error.exitCode);
   }
-  console.error('An unexpected critical error occurred:');
+  debugLogger.error('An unexpected critical error occurred:');
   if (error instanceof Error) {
-    console.error(error.stack);
+    debugLogger.error(error.stack);
   } else {
-    console.error(String(error));
+    debugLogger.error(String(error));
   }
   process.exit(1);
 });
