@@ -8,7 +8,7 @@ import { access, cp, mkdir, readdir, writeFile } from 'node:fs/promises';
 import { join, dirname, basename } from 'node:path';
 import type { CommandModule } from 'yargs';
 import { fileURLToPath } from 'node:url';
-import { getErrorMessage } from '../../utils/errors.js';
+import { debugLogger } from '@google/gemini-cli-core';
 
 interface NewArgs {
   path: string;
@@ -49,32 +49,27 @@ async function copyDirectory(template: string, path: string) {
 }
 
 async function handleNew(args: NewArgs) {
-  try {
-    if (args.template) {
-      await copyDirectory(args.template, args.path);
-      console.log(
-        `Successfully created new extension from template "${args.template}" at ${args.path}.`,
-      );
-    } else {
-      await createDirectory(args.path);
-      const extensionName = basename(args.path);
-      const manifest = {
-        name: extensionName,
-        version: '1.0.0',
-      };
-      await writeFile(
-        join(args.path, 'gemini-extension.json'),
-        JSON.stringify(manifest, null, 2),
-      );
-      console.log(`Successfully created new extension at ${args.path}.`);
-    }
-    console.log(
-      `You can install this using "gemini extensions link ${args.path}" to test it out.`,
+  if (args.template) {
+    await copyDirectory(args.template, args.path);
+    debugLogger.log(
+      `Successfully created new extension from template "${args.template}" at ${args.path}.`,
     );
-  } catch (error) {
-    console.error(getErrorMessage(error));
-    throw error;
+  } else {
+    await createDirectory(args.path);
+    const extensionName = basename(args.path);
+    const manifest = {
+      name: extensionName,
+      version: '1.0.0',
+    };
+    await writeFile(
+      join(args.path, 'gemini-extension.json'),
+      JSON.stringify(manifest, null, 2),
+    );
+    debugLogger.log(`Successfully created new extension at ${args.path}.`);
   }
+  debugLogger.log(
+    `You can install this using "gemini extensions link ${args.path}" to test it out.`,
+  );
 }
 
 async function getBoilerplateChoices() {
