@@ -7,6 +7,7 @@
 import type { CommandModule } from 'yargs';
 import { uninstallExtension } from '../../config/extension.js';
 import { getErrorMessage } from '../../utils/errors.js';
+import { debugLogger } from '@google/gemini-cli-core';
 
 interface UninstallArgs {
   name: string; // can be extension name or source URL.
@@ -14,10 +15,10 @@ interface UninstallArgs {
 
 export async function handleUninstall(args: UninstallArgs) {
   try {
-    await uninstallExtension(args.name);
-    console.log(`Extension "${args.name}" successfully uninstalled.`);
+    await uninstallExtension(args.name, false);
+    debugLogger.log(`Extension "${args.name}" successfully uninstalled.`);
   } catch (error) {
-    console.error(getErrorMessage(error));
+    debugLogger.error(getErrorMessage(error));
     process.exit(1);
   }
 }
