@@ -45,7 +45,9 @@ describe('validateAuthMethod', () => {
     it('should return an error message if GEMINI_API_KEY is not set', () => {
       vi.stubEnv('GEMINI_API_KEY', undefined);
       expect(validateAuthMethod(AuthType.USE_GEMINI)).toBe(
-        'GEMINI_API_KEY environment variable not found. Add that to your environment and try again (no reload needed if using .env)!',
+        'GEMINI_API_KEY not found. Find your existing key or generate a new one at: https://aistudio.google.com/apikey\n' +
+          '\n' +
+          'To continue, please set the GEMINI_API_KEY environment variable or add it to a .env file.',
       );
     });
   });
