@@ -15,44 +15,53 @@ import {
   Config,
   ExtensionInstallEvent,
   ExtensionUninstallEvent,
+  ExtensionUpdateEvent,
   ExtensionDisableEvent,
   ExtensionEnableEvent,
   logExtensionEnable,
   logExtensionInstallEvent,
   logExtensionUninstall,
+  logExtensionUpdateEvent,
   logExtensionDisable,
+  debugLogger,
 } from '@google/gemini-cli-core';
 import * as fs from 'node:fs';
 import * as path from 'node:path';
 import * as os from 'node:os';
 import { SettingScope, loadSettings } from '../config/settings.js';
 import { getErrorMessage } from '../utils/errors.js';
-import { recursivelyHydrateStrings } from './extensions/variables.js';
+import {
+  recursivelyHydrateStrings,
+  type JsonObject,
+} from './extensions/variables.js';
 import { isWorkspaceTrusted } from './trustedFolders.js';
 import { resolveEnvVarsInObject } from '../utils/envVarResolver.js';
-import { randomUUID } from 'node:crypto';
+import { randomUUID, createHash } from 'node:crypto';
 import {
   cloneFromGit,
   downloadFromGitHubRelease,
+  tryParseGithubUrl,
 } from './extensions/github.js';
 import type { LoadExtensionContext } from './extensions/variableSchema.js';
 import { ExtensionEnablementManager } from './extensions/extensionEnablement.js';
 import chalk from 'chalk';
 import type { ConfirmationRequest } from '../ui/types.js';
+import { escapeAnsiCtrlCodes } from '../ui/utils/textUtils.js';
 
 export const EXTENSIONS_DIRECTORY_NAME = path.join(GEMINI_DIR, 'extensions');
 
 export const EXTENSIONS_CONFIG_FILENAME = 'gemini-extension.json';
 export const INSTALL_METADATA_FILENAME = '.gemini-extension-install.json';
-
-export interface Extension {
-  path: string;
-  config: ExtensionConfig;
-  contextFiles: string[];
-  installMetadata?: ExtensionInstallMetadata | undefined;
-}
-
-export interface ExtensionConfig {
+export const INSTALL_WARNING_MESSAGE =
+  '**The extension you are about to install may have been created by a third-party developer and sourced from a public repository. Google does not vet, endorse, or guarantee the functionality or security of extensions. Please carefully inspect any extension and its source code before installing to understand the permissions it requires and the actions it may perform.**';
+/**
+ * Extension definition as written to disk in gemini-extension.json files.
+ * This should *not* be referenced outside of the logic for reading files.
+ * If information is required for manipulating extensions (load, unload, update)
+ * outside of the loading process that data needs to be stored on the
+ * GeminiCLIExtension class defined in Core.
+ */
+interface ExtensionConfig {
   name: string;
   version: string;
   mcpServers?: Record<string, MCPServerConfig>;
@@ -96,14 +105,6 @@ export class ExtensionStorage {
   }
 }
 
-export function getWorkspaceExtensions(workspaceDir: string): Extension[] {
-  // If the workspace dir is the user extensions dir, there are no workspace extensions.
-  if (path.resolve(workspaceDir) === path.resolve(os.homedir())) {
-    return [];
-  }
-  return loadExtensionsFromDir(workspaceDir);
-}
-
 export async function copyExtension(
   source: string,
   destination: string,
@@ -111,26 +112,6 @@ export async function copyExtension(
   await fs.promises.cp(source, destination, { recursive: true });
 }
 
-export async function performWorkspaceExtensionMigration(
-  extensions: Extension[],
-  requestConsent: (consent: string) => Promise<boolean>,
-): Promise<string[]> {
-  const failedInstallNames: string[] = [];
-
-  for (const extension of extensions) {
-    try {
-      const installMetadata: ExtensionInstallMetadata = {
-        source: extension.path,
-        type: 'local',
-      };
-      await installExtension(installMetadata, requestConsent);
-    } catch (_) {
-      failedInstallNames.push(extension.config.name);
-    }
-  }
-  return failedInstallNames;
-}
-
 function getTelemetryConfig(cwd: string) {
   const settings = loadSettings(cwd);
   const config = new Config({
@@ -148,66 +129,41 @@ function getTelemetryConfig(cwd: string) {
 export function loadExtensions(
   extensionEnablementManager: ExtensionEnablementManager,
   workspaceDir: string = process.cwd(),
-): Extension[] {
-  const settings = loadSettings(workspaceDir).merged;
-  const allExtensions = [...loadUserExtensions()];
-
-  if (
-    (isWorkspaceTrusted(settings) ?? true) &&
-    // Default management setting to true
-    !(settings.experimental?.extensionManagement ?? true)
-  ) {
-    allExtensions.push(...getWorkspaceExtensions(workspaceDir));
-  }
-
-  const uniqueExtensions = new Map<string, Extension>();
-
-  for (const extension of allExtensions) {
-    if (
-      !uniqueExtensions.has(extension.config.name) &&
-      extensionEnablementManager.isEnabled(extension.config.name, workspaceDir)
-    ) {
-      uniqueExtensions.set(extension.config.name, extension);
-    }
-  }
-
-  return Array.from(uniqueExtensions.values());
-}
-
-export function loadUserExtensions(): Extension[] {
-  const userExtensions = loadExtensionsFromDir(os.homedir());
-
-  const uniqueExtensions = new Map<string, Extension>();
-  for (const extension of userExtensions) {
-    if (!uniqueExtensions.has(extension.config.name)) {
-      uniqueExtensions.set(extension.config.name, extension);
-    }
-  }
-
-  return Array.from(uniqueExtensions.values());
-}
-
-export function loadExtensionsFromDir(dir: string): Extension[] {
-  const storage = new Storage(dir);
-  const extensionsDir = storage.getExtensionsDir();
+): GeminiCLIExtension[] {
+  const extensionsDir = ExtensionStorage.getUserExtensionsDir();
   if (!fs.existsSync(extensionsDir)) {
     return [];
   }
 
-  const extensions: Extension[] = [];
+  const extensions: GeminiCLIExtension[] = [];
   for (const subdir of fs.readdirSync(extensionsDir)) {
     const extensionDir = path.join(extensionsDir, subdir);
 
-    const extension = loadExtension({ extensionDir, workspaceDir: dir });
+    const extension = loadExtension({
+      extensionDir,
+      workspaceDir,
+      extensionEnablementManager,
+    });
     if (extension != null) {
       extensions.push(extension);
     }
   }
-  return extensions;
+
+  const uniqueExtensions = new Map<string, GeminiCLIExtension>();
+
+  for (const extension of extensions) {
+    if (!uniqueExtensions.has(extension.name)) {
+      uniqueExtensions.set(extension.name, extension);
+    }
+  }
+
+  return Array.from(uniqueExtensions.values());
 }
 
-export function loadExtension(context: LoadExtensionContext): Extension | null {
-  const { extensionDir, workspaceDir } = context;
+export function loadExtension(
+  context: LoadExtensionContext,
+): GeminiCLIExtension | null {
+  const { extensionDir, workspaceDir, extensionEnablementManager } = context;
   if (!fs.statSync(extensionDir).isDirectory()) {
     return null;
   }
@@ -223,6 +179,7 @@ export function loadExtension(context: LoadExtensionContext): Extension | null {
     let config = loadExtensionConfig({
       extensionDir: effectiveExtensionPath,
       workspaceDir,
+      extensionEnablementManager,
     });
 
     config = resolveEnvVarsInObject(config);
@@ -243,13 +200,18 @@ export function loadExtension(context: LoadExtensionContext): Extension | null {
       .filter((contextFilePath) => fs.existsSync(contextFilePath));
 
     return {
+      name: config.name,
+      version: config.version,
       path: effectiveExtensionPath,
-      config,
       contextFiles,
       installMetadata,
+      mcpServers: config.mcpServers,
+      excludeTools: config.excludeTools,
+      isActive: extensionEnablementManager.isEnabled(config.name, workspaceDir),
+      id: getExtensionId(config, installMetadata),
     };
   } catch (e) {
-    console.error(
+    debugLogger.error(
       `Warning: Skipping extension in ${effectiveExtensionPath}: ${getErrorMessage(
         e,
       )}`,
@@ -260,8 +222,9 @@ export function loadExtension(context: LoadExtensionContext): Extension | null {
 
 export function loadExtensionByName(
   name: string,
+  extensionEnablementManager: ExtensionEnablementManager,
   workspaceDir: string = process.cwd(),
-): Extension | null {
+): GeminiCLIExtension | null {
   const userExtensionsDir = ExtensionStorage.getUserExtensionsDir();
   if (!fs.existsSync(userExtensionsDir)) {
     return null;
@@ -272,11 +235,12 @@ export function loadExtensionByName(
     if (!fs.statSync(extensionDir).isDirectory()) {
       continue;
     }
-    const extension = loadExtension({ extensionDir, workspaceDir });
-    if (
-      extension &&
-      extension.config.name.toLowerCase() === name.toLowerCase()
-    ) {
+    const extension = loadExtension({
+      extensionDir,
+      workspaceDir,
+      extensionEnablementManager,
+    });
+    if (extension && extension.name.toLowerCase() === name.toLowerCase()) {
       return extension;
     }
   }
@@ -312,28 +276,6 @@ function getContextFileNames(config: ExtensionConfig): string[] {
   return config.contextFileName;
 }
 
-/**
- * Returns an annotated list of extensions. If an extension is listed in enabledExtensionNames, it will be active.
- * If enabledExtensionNames is empty, an extension is active unless it is disabled.
- * @param extensions The base list of extensions.
- * @param enabledExtensionNames The names of explicitly enabled extensions.
- * @param workspaceDir The current workspace directory.
- */
-export function annotateActiveExtensions(
-  extensions: Extension[],
-  workspaceDir: string,
-  manager: ExtensionEnablementManager,
-): GeminiCLIExtension[] {
-  manager.validateExtensionOverrides(extensions);
-  return extensions.map((extension) => ({
-    name: extension.config.name,
-    version: extension.config.version,
-    isActive: manager.isEnabled(extension.config.name, workspaceDir),
-    path: extension.path,
-    installMetadata: extension.installMetadata,
-  }));
-}
-
 /**
  * Requests consent from the user to perform an action, by reading a Y/n
  * character from stdin.
@@ -346,7 +288,7 @@ export function annotateActiveExtensions(
 export async function requestConsentNonInteractive(
   consentDescription: string,
 ): Promise<boolean> {
-  console.info(consentDescription);
+  debugLogger.log(consentDescription);
   const result = await promptForConsentNonInteractive(
     'Do you want to continue? [Y/n]: ',
   );
@@ -420,19 +362,25 @@ async function promptForConsentInteractive(
   });
 }
 
-export async function installExtension(
+export function hashValue(value: string): string {
+  return createHash('sha256').update(value).digest('hex');
+}
+
+export async function installOrUpdateExtension(
   installMetadata: ExtensionInstallMetadata,
   requestConsent: (consent: string) => Promise<boolean>,
   cwd: string = process.cwd(),
   previousExtensionConfig?: ExtensionConfig,
 ): Promise<string> {
+  const isUpdate = !!previousExtensionConfig;
   const telemetryConfig = getTelemetryConfig(cwd);
   let newExtensionConfig: ExtensionConfig | null = null;
   let localSourcePath: string | undefined;
+  const extensionEnablementManager = new ExtensionEnablementManager();
 
   try {
     const settings = loadSettings(cwd).merged;
-    if (!isWorkspaceTrusted(settings)) {
+    if (!isWorkspaceTrusted(settings).isTrusted) {
       throw new Error(
         `Could not install extension from untrusted folder at ${installMetadata.source}`,
       );
@@ -455,16 +403,36 @@ export async function installExtension(
       installMetadata.type === 'github-release'
     ) {
       tempDir = await ExtensionStorage.createTmpDir();
-      try {
+      const parsedGithubParts = tryParseGithubUrl(installMetadata.source);
+      if (!parsedGithubParts) {
+        await cloneFromGit(installMetadata, tempDir);
+        installMetadata.type = 'git';
+      } else {
         const result = await downloadFromGitHubRelease(
           installMetadata,
           tempDir,
+          parsedGithubParts,
         );
-        installMetadata.type = result.type;
-        installMetadata.releaseTag = result.tagName;
-      } catch (_error) {
-        await cloneFromGit(installMetadata, tempDir);
-        installMetadata.type = 'git';
+        if (result.success) {
+          installMetadata.type = result.type;
+          installMetadata.releaseTag = result.tagName;
+        } else if (
+          // This repo has no github releases, and wasn't explicitly installed
+          // from a github release, unconditionally just clone it.
+          (result.failureReason === 'no release data' &&
+            installMetadata.type === 'git') ||
+          // Otherwise ask the user if they would like to try a git clone.
+          (await requestConsent(
+            `Error downloading github release for ${installMetadata.source} with the following error: ${result.errorMessage}.\n\nWould you like to attempt to install via "git clone" instead?`,
+          ))
+        ) {
+          await cloneFromGit(installMetadata, tempDir);
+          installMetadata.type = 'git';
+        } else {
+          throw new Error(
+            `Failed to install extension ${installMetadata.source}: ${result.errorMessage}`,
+          );
+        }
       }
       localSourcePath = tempDir;
     } else if (
@@ -480,27 +448,39 @@ export async function installExtension(
       newExtensionConfig = loadExtensionConfig({
         extensionDir: localSourcePath,
         workspaceDir: cwd,
+        extensionEnablementManager,
       });
 
       const newExtensionName = newExtensionConfig.name;
-      const extensionStorage = new ExtensionStorage(newExtensionName);
-      const destinationPath = extensionStorage.getExtensionDir();
-
-      const installedExtensions = loadUserExtensions();
-      if (
-        installedExtensions.some(
-          (installed) => installed.config.name === newExtensionName,
-        )
-      ) {
-        throw new Error(
-          `Extension "${newExtensionName}" is already installed. Please uninstall it first.`,
+      if (!isUpdate) {
+        const installedExtensions = loadExtensions(
+          new ExtensionEnablementManager(),
+          cwd,
         );
+        if (
+          installedExtensions.some(
+            (installed) => installed.name === newExtensionName,
+          )
+        ) {
+          throw new Error(
+            `Extension "${newExtensionName}" is already installed. Please uninstall it first.`,
+          );
+        }
       }
+
       await maybeRequestConsentOrFail(
         newExtensionConfig,
         requestConsent,
         previousExtensionConfig,
       );
+
+      const extensionStorage = new ExtensionStorage(newExtensionName);
+      const destinationPath = extensionStorage.getExtensionDir();
+
+      if (isUpdate) {
+        await uninstallExtension(newExtensionName, isUpdate, cwd);
+      }
+
       await fs.promises.mkdir(destinationPath, { recursive: true });
 
       if (
@@ -522,18 +502,36 @@ export async function installExtension(
         await fs.promises.rm(tempDir, { recursive: true, force: true });
       }
     }
+    if (isUpdate) {
+      logExtensionUpdateEvent(
+        telemetryConfig,
+        new ExtensionUpdateEvent(
+          hashValue(newExtensionConfig.name),
+          getExtensionId(newExtensionConfig, installMetadata),
+          newExtensionConfig.version,
+          previousExtensionConfig.version,
+          installMetadata.type,
+          'success',
+        ),
+      );
+    } else {
+      logExtensionInstallEvent(
+        telemetryConfig,
+        new ExtensionInstallEvent(
+          hashValue(newExtensionConfig.name),
+          getExtensionId(newExtensionConfig, installMetadata),
+          newExtensionConfig.version,
+          installMetadata.type,
+          'success',
+        ),
+      );
+      enableExtension(
+        newExtensionConfig.name,
+        SettingScope.User,
+        extensionEnablementManager,
+      );
+    }
 
-    logExtensionInstallEvent(
-      telemetryConfig,
-      new ExtensionInstallEvent(
-        newExtensionConfig!.name,
-        newExtensionConfig!.version,
-        installMetadata.source,
-        'success',
-      ),
-    );
-
-    enableExtension(newExtensionConfig!.name, SettingScope.User);
     return newExtensionConfig!.name;
   } catch (error) {
     // Attempt to load config from the source path even if installation fails
@@ -543,20 +541,40 @@ export async function installExtension(
         newExtensionConfig = loadExtensionConfig({
           extensionDir: localSourcePath,
           workspaceDir: cwd,
+          extensionEnablementManager,
         });
       } catch {
         // Ignore error, this is just for logging.
       }
     }
-    logExtensionInstallEvent(
-      telemetryConfig,
-      new ExtensionInstallEvent(
-        newExtensionConfig?.name ?? '',
-        newExtensionConfig?.version ?? '',
-        installMetadata.source,
-        'error',
-      ),
-    );
+    const config = newExtensionConfig ?? previousExtensionConfig;
+    const extensionId = config
+      ? getExtensionId(config, installMetadata)
+      : undefined;
+    if (isUpdate) {
+      logExtensionUpdateEvent(
+        telemetryConfig,
+        new ExtensionUpdateEvent(
+          hashValue(config?.name ?? ''),
+          extensionId ?? '',
+          newExtensionConfig?.version ?? '',
+          previousExtensionConfig.version,
+          installMetadata.type,
+          'error',
+        ),
+      );
+    } else {
+      logExtensionInstallEvent(
+        telemetryConfig,
+        new ExtensionInstallEvent(
+          hashValue(newExtensionConfig?.name ?? ''),
+          extensionId ?? '',
+          newExtensionConfig?.version ?? '',
+          installMetadata.type,
+          'error',
+        ),
+      );
+    }
     throw error;
   }
 }
@@ -566,12 +584,11 @@ export async function installExtension(
  * extensionConfig.
  */
 function extensionConsentString(extensionConfig: ExtensionConfig): string {
+  const sanitizedConfig = escapeAnsiCtrlCodes(extensionConfig);
   const output: string[] = [];
-  const mcpServerEntries = Object.entries(extensionConfig.mcpServers || {});
-  output.push(`Installing extension "${extensionConfig.name}".`);
-  output.push(
-    '**Extensions may introduce unexpected behavior. Ensure you have investigated the extension source and trust the author.**',
-  );
+  const mcpServerEntries = Object.entries(sanitizedConfig.mcpServers || {});
+  output.push(`Installing extension "${sanitizedConfig.name}".`);
+  output.push(INSTALL_WARNING_MESSAGE);
 
   if (mcpServerEntries.length) {
     output.push('This extension will run the following MCP servers:');
@@ -583,14 +600,14 @@ function extensionConsentString(extensionConfig: ExtensionConfig): string {
       output.push(`  * ${key} (${isLocal ? 'local' : 'remote'}): ${source}`);
     }
   }
-  if (extensionConfig.contextFileName) {
+  if (sanitizedConfig.contextFileName) {
     output.push(
-      `This extension will append info to your gemini.md context using ${extensionConfig.contextFileName}`,
+      `This extension will append info to your gemini.md context using ${sanitizedConfig.contextFileName}`,
     );
   }
-  if (extensionConfig.excludeTools) {
+  if (sanitizedConfig.excludeTools) {
     output.push(
-      `This extension will exclude the following core tools: ${extensionConfig.excludeTools}`,
+      `This extension will exclude the following core tools: ${sanitizedConfig.excludeTools}`,
     );
   }
   return output.join('\n');
@@ -642,17 +659,23 @@ export function loadExtensionConfig(
   }
   try {
     const configContent = fs.readFileSync(configFilePath, 'utf-8');
-    const config = recursivelyHydrateStrings(JSON.parse(configContent), {
-      extensionPath: extensionDir,
-      workspacePath: workspaceDir,
-      '/': path.sep,
-      pathSeparator: path.sep,
-    }) as unknown as ExtensionConfig;
-    if (!config.name || !config.version) {
+    const rawConfig = JSON.parse(configContent) as ExtensionConfig;
+    if (!rawConfig.name || !rawConfig.version) {
       throw new Error(
-        `Invalid configuration in ${configFilePath}: missing ${!config.name ? '"name"' : '"version"'}`,
+        `Invalid configuration in ${configFilePath}: missing ${!rawConfig.name ? '"name"' : '"version"'}`,
       );
     }
+    const installDir = new ExtensionStorage(rawConfig.name).getExtensionDir();
+    const config = recursivelyHydrateStrings(
+      rawConfig as unknown as JsonObject,
+      {
+        extensionPath: installDir,
+        workspacePath: workspaceDir,
+        '/': path.sep,
+        pathSeparator: path.sep,
+      },
+    ) as unknown as ExtensionConfig;
+
     validateName(config.name);
     return config;
   } catch (e) {
@@ -666,52 +689,58 @@ export function loadExtensionConfig(
 
 export async function uninstallExtension(
   extensionIdentifier: string,
+  isUpdate: boolean,
   cwd: string = process.cwd(),
 ): Promise<void> {
-  const telemetryConfig = getTelemetryConfig(cwd);
-  const installedExtensions = loadUserExtensions();
-  const extensionName = installedExtensions.find(
+  const installedExtensions = loadExtensions(
+    new ExtensionEnablementManager(),
+    cwd,
+  );
+  const extension = installedExtensions.find(
     (installed) =>
-      installed.config.name.toLowerCase() ===
-        extensionIdentifier.toLowerCase() ||
+      installed.name.toLowerCase() === extensionIdentifier.toLowerCase() ||
       installed.installMetadata?.source.toLowerCase() ===
         extensionIdentifier.toLowerCase(),
-  )?.config.name;
-  if (!extensionName) {
+  );
+  if (!extension) {
     throw new Error(`Extension not found.`);
   }
-  const manager = new ExtensionEnablementManager(
-    ExtensionStorage.getUserExtensionsDir(),
-    [extensionName],
-  );
-  manager.remove(extensionName);
-  const storage = new ExtensionStorage(extensionName);
+  const storage = new ExtensionStorage(extension.name);
 
   await fs.promises.rm(storage.getExtensionDir(), {
     recursive: true,
     force: true,
   });
+
+  // The rest of the cleanup below here is only for true uninstalls, not
+  // uninstalls related to updates.
+  if (isUpdate) return;
+
+  const manager = new ExtensionEnablementManager([extension.name]);
+  manager.remove(extension.name);
+
+  const telemetryConfig = getTelemetryConfig(cwd);
   logExtensionUninstall(
     telemetryConfig,
-    new ExtensionUninstallEvent(extensionName, 'success'),
+    new ExtensionUninstallEvent(
+      hashValue(extension.name),
+      extension.id,
+      'success',
+    ),
   );
 }
 
 export function toOutputString(
-  extension: Extension,
+  extension: GeminiCLIExtension,
   workspaceDir: string,
 ): string {
-  const manager = new ExtensionEnablementManager(
-    ExtensionStorage.getUserExtensionsDir(),
-  );
-  const userEnabled = manager.isEnabled(extension.config.name, os.homedir());
-  const workspaceEnabled = manager.isEnabled(
-    extension.config.name,
-    workspaceDir,
-  );
+  const manager = new ExtensionEnablementManager();
+  const userEnabled = manager.isEnabled(extension.name, os.homedir());
+  const workspaceEnabled = manager.isEnabled(extension.name, workspaceDir);
 
   const status = workspaceEnabled ? chalk.green('✓') : chalk.red('✗');
-  let output = `${status} ${extension.config.name} (${extension.config.version})`;
+  let output = `${status} ${extension.name} (${extension.version})`;
+  output += `\n ID: ${extension.id}`;
   output += `\n Path: ${extension.path}`;
   if (extension.installMetadata) {
     output += `\n Source: ${extension.installMetadata.source} (Type: ${extension.installMetadata.type})`;
@@ -730,15 +759,15 @@ export function toOutputString(
       output += `\n  ${contextFile}`;
     });
   }
-  if (extension.config.mcpServers) {
+  if (extension.mcpServers) {
     output += `\n MCP servers:`;
-    Object.keys(extension.config.mcpServers).forEach((key) => {
+    Object.keys(extension.mcpServers).forEach((key) => {
       output += `\n  ${key}`;
     });
   }
-  if (extension.config.excludeTools) {
+  if (extension.excludeTools) {
     output += `\n Excluded tools:`;
-    extension.config.excludeTools.forEach((tool) => {
+    extension.excludeTools.forEach((tool) => {
       output += `\n  ${tool}`;
     });
   }
@@ -748,43 +777,68 @@ export function toOutputString(
 export function disableExtension(
   name: string,
   scope: SettingScope,
+  extensionEnablementManager: ExtensionEnablementManager,
   cwd: string = process.cwd(),
 ) {
   const config = getTelemetryConfig(cwd);
   if (scope === SettingScope.System || scope === SettingScope.SystemDefaults) {
     throw new Error('System and SystemDefaults scopes are not supported.');
   }
-  const extension = loadExtensionByName(name, cwd);
+  const extension = loadExtensionByName(name, extensionEnablementManager, cwd);
   if (!extension) {
     throw new Error(`Extension with name ${name} does not exist.`);
   }
 
-  const manager = new ExtensionEnablementManager(
-    ExtensionStorage.getUserExtensionsDir(),
-    [name],
-  );
   const scopePath = scope === SettingScope.Workspace ? cwd : os.homedir();
-  manager.disable(name, true, scopePath);
-  logExtensionDisable(config, new ExtensionDisableEvent(name, scope));
+  extensionEnablementManager.disable(name, true, scopePath);
+  logExtensionDisable(
+    config,
+    new ExtensionDisableEvent(hashValue(name), extension.id, scope),
+  );
 }
 
 export function enableExtension(
   name: string,
   scope: SettingScope,
+  extensionEnablementManager: ExtensionEnablementManager,
   cwd: string = process.cwd(),
 ) {
   if (scope === SettingScope.System || scope === SettingScope.SystemDefaults) {
     throw new Error('System and SystemDefaults scopes are not supported.');
   }
-  const extension = loadExtensionByName(name, cwd);
+  const extension = loadExtensionByName(name, extensionEnablementManager, cwd);
   if (!extension) {
     throw new Error(`Extension with name ${name} does not exist.`);
   }
-  const manager = new ExtensionEnablementManager(
-    ExtensionStorage.getUserExtensionsDir(),
-  );
   const scopePath = scope === SettingScope.Workspace ? cwd : os.homedir();
-  manager.enable(name, true, scopePath);
+  extensionEnablementManager.enable(name, true, scopePath);
   const config = getTelemetryConfig(cwd);
-  logExtensionEnable(config, new ExtensionEnableEvent(name, scope));
+  logExtensionEnable(
+    config,
+    new ExtensionEnableEvent(hashValue(name), extension.id, scope),
+  );
+}
+
+function getExtensionId(
+  config: ExtensionConfig,
+  installMetadata?: ExtensionInstallMetadata,
+): string {
+  // IDs are created by hashing details of the installation source in order to
+  // deduplicate extensions with conflicting names and also obfuscate any
+  // potentially sensitive information such as private git urls, system paths,
+  // or project names.
+  let idValue = config.name;
+  const githubUrlParts =
+    installMetadata &&
+    (installMetadata.type === 'git' ||
+      installMetadata.type === 'github-release')
+      ? tryParseGithubUrl(installMetadata.source)
+      : null;
+  if (githubUrlParts) {
+    // For github repos, we use the https URI to the repo as the ID.
+    idValue = `https://github.com/${githubUrlParts.owner}/${githubUrlParts.repo}`;
+  } else {
+    idValue = installMetadata?.source ?? config.name;
+  }
+  return hashValue(idValue);
 }
