@@ -6,7 +6,8 @@
 
 import fs from 'node:fs';
 import path from 'node:path';
-import { type Extension } from '../extension.js';
+import type { GeminiCLIExtension } from '@google/gemini-cli-core';
+import { ExtensionStorage } from '../extension.js';
 
 export interface ExtensionEnablementConfig {
   overrides: string[];
@@ -112,20 +113,21 @@ export class ExtensionEnablementManager {
   // only the ones in this list.
   private enabledExtensionNamesOverride: string[];
 
-  constructor(configDir: string, enabledExtensionNames?: string[]) {
-    this.configDir = configDir;
-    this.configFilePath = path.join(configDir, 'extension-enablement.json');
+  constructor(enabledExtensionNames?: string[]) {
+    this.configDir = ExtensionStorage.getUserExtensionsDir();
+    this.configFilePath = path.join(
+      this.configDir,
+      'extension-enablement.json',
+    );
     this.enabledExtensionNamesOverride =
       enabledExtensionNames?.map((name) => name.toLowerCase()) ?? [];
   }
 
-  validateExtensionOverrides(extensions: Extension[]) {
+  validateExtensionOverrides(extensions: GeminiCLIExtension[]) {
     for (const name of this.enabledExtensionNamesOverride) {
       if (name === 'none') continue;
       if (
-        !extensions.some(
-          (ext) => ext.config.name.toLowerCase() === name.toLowerCase(),
-        )
+        !extensions.some((ext) => ext.name.toLowerCase() === name.toLowerCase())
       ) {
         console.error(`Extension not found: ${name}`);
       }
