@@ -10,7 +10,8 @@ import {
   cloneFromGit,
   extractFile,
   findReleaseAsset,
-  parseGitHubRepoForReleases,
+  fetchReleaseFromGithub,
+  tryParseGithubUrl,
 } from './github.js';
 import { simpleGit, type SimpleGit } from 'simple-git';
 import { ExtensionUpdateState } from '../../ui/state/extensions.js';
@@ -21,6 +22,7 @@ import * as path from 'node:path';
 import * as tar from 'tar';
 import * as archiver from 'archiver';
 import type { GeminiCLIExtension } from '@google/gemini-cli-core';
+import { ExtensionEnablementManager } from './extensionEnablement.js';
 
 const mockPlatform = vi.hoisted(() => vi.fn());
 const mockArch = vi.hoisted(() => vi.fn());
@@ -35,6 +37,15 @@ vi.mock('node:os', async (importOriginal) => {
 
 vi.mock('simple-git');
 
+const fetchJsonMock = vi.hoisted(() => vi.fn());
+vi.mock('./github_fetch.js', async (importOriginal) => {
+  const actual = await importOriginal<typeof import('./github_fetch.js')>();
+  return {
+    ...actual,
+    fetchJson: fetchJsonMock,
+  };
+});
+
 describe('git extension helpers', () => {
   afterEach(() => {
     vi.restoreAllMocks();
@@ -130,6 +141,7 @@ describe('git extension helpers', () => {
     it('should return NOT_UPDATABLE for non-git extensions', async () => {
       const extension: GeminiCLIExtension = {
         name: 'test',
+        id: 'test-id',
         path: '/ext',
         version: '1.0.0',
         isActive: true,
@@ -137,11 +149,11 @@ describe('git extension helpers', () => {
           type: 'link',
           source: '',
         },
+        contextFiles: [],
       };
-      let result: ExtensionUpdateState | undefined = undefined;
-      await checkForExtensionUpdate(
+      const result = await checkForExtensionUpdate(
         extension,
-        (newState) => (result = newState),
+        new ExtensionEnablementManager(),
       );
       expect(result).toBe(ExtensionUpdateState.NOT_UPDATABLE);
     });
@@ -149,6 +161,7 @@ describe('git extension helpers', () => {
     it('should return ERROR if no remotes found', async () => {
       const extension: GeminiCLIExtension = {
         name: 'test',
+        id: 'test-id',
         path: '/ext',
         version: '1.0.0',
         isActive: true,
@@ -156,12 +169,12 @@ describe('git extension helpers', () => {
           type: 'git',
           source: '',
         },
+        contextFiles: [],
       };
       mockGit.getRemotes.mockResolvedValue([]);
-      let result: ExtensionUpdateState | undefined = undefined;
-      await checkForExtensionUpdate(
+      const result = await checkForExtensionUpdate(
         extension,
-        (newState) => (result = newState),
+        new ExtensionEnablementManager(),
       );
       expect(result).toBe(ExtensionUpdateState.ERROR);
     });
@@ -169,6 +182,7 @@ describe('git extension helpers', () => {
     it('should return UPDATE_AVAILABLE when remote hash is different', async () => {
       const extension: GeminiCLIExtension = {
         name: 'test',
+        id: 'test-id',
         path: '/ext',
         version: '1.0.0',
         isActive: true,
@@ -176,6 +190,7 @@ describe('git extension helpers', () => {
           type: 'git',
           source: 'my/ext',
         },
+        contextFiles: [],
       };
       mockGit.getRemotes.mockResolvedValue([
         { name: 'origin', refs: { fetch: 'http://my-repo.com' } },
@@ -183,10 +198,9 @@ describe('git extension helpers', () => {
       mockGit.listRemote.mockResolvedValue('remote-hash\tHEAD');
       mockGit.revparse.mockResolvedValue('local-hash');
 
-      let result: ExtensionUpdateState | undefined = undefined;
-      await checkForExtensionUpdate(
+      const result = await checkForExtensionUpdate(
         extension,
-        (newState) => (result = newState),
+        new ExtensionEnablementManager(),
       );
       expect(result).toBe(ExtensionUpdateState.UPDATE_AVAILABLE);
     });
@@ -194,6 +208,7 @@ describe('git extension helpers', () => {
     it('should return UP_TO_DATE when remote and local hashes are the same', async () => {
       const extension: GeminiCLIExtension = {
         name: 'test',
+        id: 'test-id',
         path: '/ext',
         version: '1.0.0',
         isActive: true,
@@ -201,6 +216,7 @@ describe('git extension helpers', () => {
           type: 'git',
           source: 'my/ext',
         },
+        contextFiles: [],
       };
       mockGit.getRemotes.mockResolvedValue([
         { name: 'origin', refs: { fetch: 'http://my-repo.com' } },
@@ -208,10 +224,9 @@ describe('git extension helpers', () => {
       mockGit.listRemote.mockResolvedValue('same-hash\tHEAD');
       mockGit.revparse.mockResolvedValue('same-hash');
 
-      let result: ExtensionUpdateState | undefined = undefined;
-      await checkForExtensionUpdate(
+      const result = await checkForExtensionUpdate(
         extension,
-        (newState) => (result = newState),
+        new ExtensionEnablementManager(),
       );
       expect(result).toBe(ExtensionUpdateState.UP_TO_DATE);
     });
@@ -219,6 +234,7 @@ describe('git extension helpers', () => {
     it('should return ERROR on git error', async () => {
       const extension: GeminiCLIExtension = {
         name: 'test',
+        id: 'test-id',
         path: '/ext',
         version: '1.0.0',
         isActive: true,
@@ -226,18 +242,78 @@ describe('git extension helpers', () => {
           type: 'git',
           source: 'my/ext',
         },
+        contextFiles: [],
       };
       mockGit.getRemotes.mockRejectedValue(new Error('git error'));
 
-      let result: ExtensionUpdateState | undefined = undefined;
-      await checkForExtensionUpdate(
+      const result = await checkForExtensionUpdate(
         extension,
-        (newState) => (result = newState),
+        new ExtensionEnablementManager(),
       );
       expect(result).toBe(ExtensionUpdateState.ERROR);
     });
   });
 
+  describe('fetchReleaseFromGithub', () => {
+    it('should fetch the latest release if allowPreRelease is true', async () => {
+      const releases = [{ tag_name: 'v1.0.0-alpha' }, { tag_name: 'v0.9.0' }];
+      fetchJsonMock.mockResolvedValueOnce(releases);
+
+      const result = await fetchReleaseFromGithub(
+        'owner',
+        'repo',
+        undefined,
+        true,
+      );
+
+      expect(fetchJsonMock).toHaveBeenCalledWith(
+        'https://api.github.com/repos/owner/repo/releases?per_page=1',
+      );
+      expect(result).toEqual(releases[0]);
+    });
+
+    it('should fetch the latest release if allowPreRelease is false', async () => {
+      const release = { tag_name: 'v0.9.0' };
+      fetchJsonMock.mockResolvedValueOnce(release);
+
+      const result = await fetchReleaseFromGithub(
+        'owner',
+        'repo',
+        undefined,
+        false,
+      );
+
+      expect(fetchJsonMock).toHaveBeenCalledWith(
+        'https://api.github.com/repos/owner/repo/releases/latest',
+      );
+      expect(result).toEqual(release);
+    });
+
+    it('should fetch a release by tag if ref is provided', async () => {
+      const release = { tag_name: 'v0.9.0' };
+      fetchJsonMock.mockResolvedValueOnce(release);
+
+      const result = await fetchReleaseFromGithub('owner', 'repo', 'v0.9.0');
+
+      expect(fetchJsonMock).toHaveBeenCalledWith(
+        'https://api.github.com/repos/owner/repo/releases/tags/v0.9.0',
+      );
+      expect(result).toEqual(release);
+    });
+
+    it('should fetch latest stable release if allowPreRelease is undefined', async () => {
+      const release = { tag_name: 'v0.9.0' };
+      fetchJsonMock.mockResolvedValueOnce(release);
+
+      const result = await fetchReleaseFromGithub('owner', 'repo');
+
+      expect(fetchJsonMock).toHaveBeenCalledWith(
+        'https://api.github.com/repos/owner/repo/releases/latest',
+      );
+      expect(result).toEqual(release);
+    });
+  });
+
   describe('findReleaseAsset', () => {
     const assets = [
       { name: 'darwin.arm64.extension.tar.gz', browser_download_url: 'url1' },
@@ -293,56 +369,62 @@ describe('git extension helpers', () => {
   describe('parseGitHubRepoForReleases', () => {
     it('should parse owner and repo from a full GitHub URL', () => {
       const source = 'https://github.com/owner/repo.git';
-      const { owner, repo } = parseGitHubRepoForReleases(source);
+      const { owner, repo } = tryParseGithubUrl(source)!;
       expect(owner).toBe('owner');
       expect(repo).toBe('repo');
     });
 
-    it('should parse owner and repo from a full GitHub UR without .git', () => {
+    it('should parse owner and repo from a full GitHub URL without .git', () => {
       const source = 'https://github.com/owner/repo';
-      const { owner, repo } = parseGitHubRepoForReleases(source);
+      const { owner, repo } = tryParseGithubUrl(source)!;
       expect(owner).toBe('owner');
       expect(repo).toBe('repo');
     });
 
-    it('should fail on a GitHub SSH URL', () => {
-      const source = 'git@github.com:owner/repo.git';
-      expect(() => parseGitHubRepoForReleases(source)).toThrow(
-        'GitHub release-based extensions are not supported for SSH. You must use an HTTPS URI with a personal access token to download releases from private repositories. You can set your personal access token in the GITHUB_TOKEN environment variable and install the extension via SSH.',
-      );
+    it('should parse owner and repo from a full GitHub URL with a trailing slash', () => {
+      const source = 'https://github.com/owner/repo/';
+      const { owner, repo } = tryParseGithubUrl(source)!;
+      expect(owner).toBe('owner');
+      expect(repo).toBe('repo');
     });
 
-    it('should fail on a non-GitHub URL', () => {
+    it('should parse owner and repo from a GitHub SSH URL', () => {
+      const source = 'git@github.com:owner/repo.git';
+
+      const { owner, repo } = tryParseGithubUrl(source)!;
+      expect(owner).toBe('owner');
+      expect(repo).toBe('repo');
+    });
+
+    it('should return null on a non-GitHub URL', () => {
       const source = 'https://example.com/owner/repo.git';
-      expect(() => parseGitHubRepoForReleases(source)).toThrow(
-        'Invalid GitHub repository source: https://example.com/owner/repo.git. Expected "owner/repo" or a github repo uri.',
-      );
+      expect(tryParseGithubUrl(source)).toBe(null);
     });
 
     it('should parse owner and repo from a shorthand string', () => {
       const source = 'owner/repo';
-      const { owner, repo } = parseGitHubRepoForReleases(source);
+      const { owner, repo } = tryParseGithubUrl(source)!;
       expect(owner).toBe('owner');
       expect(repo).toBe('repo');
     });
 
     it('should handle .git suffix in repo name', () => {
       const source = 'owner/repo.git';
-      const { owner, repo } = parseGitHubRepoForReleases(source);
+      const { owner, repo } = tryParseGithubUrl(source)!;
       expect(owner).toBe('owner');
       expect(repo).toBe('repo');
     });
 
     it('should throw error for invalid source format', () => {
       const source = 'invalid-format';
-      expect(() => parseGitHubRepoForReleases(source)).toThrow(
+      expect(() => tryParseGithubUrl(source)).toThrow(
         'Invalid GitHub repository source: invalid-format. Expected "owner/repo" or a github repo uri.',
       );
     });
 
     it('should throw error for source with too many parts', () => {
       const source = 'https://github.com/owner/repo/extra';
-      expect(() => parseGitHubRepoForReleases(source)).toThrow(
+      expect(() => tryParseGithubUrl(source)).toThrow(
         'Invalid GitHub repository source: https://github.com/owner/repo/extra. Expected "owner/repo" or a github repo uri.',
       );
     });
