@@ -11,17 +11,17 @@ import {
 } from '../../ui/state/extensions.js';
 import {
   copyExtension,
-  installExtension,
-  uninstallExtension,
+  installOrUpdateExtension,
   loadExtension,
   loadInstallMetadata,
   ExtensionStorage,
   loadExtensionConfig,
 } from '../extension.js';
 import { checkForExtensionUpdate } from './github.js';
-import type { GeminiCLIExtension } from '@google/gemini-cli-core';
+import { debugLogger, type GeminiCLIExtension } from '@google/gemini-cli-core';
 import * as fs from 'node:fs';
 import { getErrorMessage } from '../../utils/errors.js';
+import { type ExtensionEnablementManager } from './extensionEnablement.js';
 
 export interface ExtensionUpdateInfo {
   name: string;
@@ -31,6 +31,7 @@ export interface ExtensionUpdateInfo {
 
 export async function updateExtension(
   extension: GeminiCLIExtension,
+  extensionEnablementManager: ExtensionEnablementManager,
   cwd: string = process.cwd(),
   requestConsent: (consent: string) => Promise<boolean>,
   currentState: ExtensionUpdateState,
@@ -65,13 +66,12 @@ export async function updateExtension(
 
   const tempDir = await ExtensionStorage.createTmpDir();
   try {
-    await copyExtension(extension.path, tempDir);
     const previousExtensionConfig = await loadExtensionConfig({
       extensionDir: extension.path,
       workspaceDir: cwd,
+      extensionEnablementManager,
     });
-    await uninstallExtension(extension.name, cwd);
-    await installExtension(
+    await installOrUpdateExtension(
       installMetadata,
       requestConsent,
       cwd,
@@ -82,6 +82,7 @@ export async function updateExtension(
     const updatedExtension = loadExtension({
       extensionDir: updatedExtensionStorage.getExtensionDir(),
       workspaceDir: cwd,
+      extensionEnablementManager,
     });
     if (!updatedExtension) {
       dispatchExtensionStateUpdate({
@@ -90,7 +91,7 @@ export async function updateExtension(
       });
       throw new Error('Updated extension not found after installation.');
     }
-    const updatedVersion = updatedExtension.config.version;
+    const updatedVersion = updatedExtension.version;
     dispatchExtensionStateUpdate({
       type: 'SET_STATE',
       payload: {
@@ -104,7 +105,7 @@ export async function updateExtension(
       updatedVersion,
     };
   } catch (e) {
-    console.error(
+    debugLogger.error(
       `Error updating extension, rolling back. ${getErrorMessage(e)}`,
     );
     dispatchExtensionStateUpdate({
@@ -123,6 +124,7 @@ export async function updateAllUpdatableExtensions(
   requestConsent: (consent: string) => Promise<boolean>,
   extensions: GeminiCLIExtension[],
   extensionsState: Map<string, ExtensionUpdateStatus>,
+  extensionEnablementManager: ExtensionEnablementManager,
   dispatch: (action: ExtensionUpdateAction) => void,
 ): Promise<ExtensionUpdateInfo[]> {
   return (
@@ -136,6 +138,7 @@ export async function updateAllUpdatableExtensions(
         .map((extension) =>
           updateExtension(
             extension,
+            extensionEnablementManager,
             cwd,
             requestConsent,
             extensionsState.get(extension.name)!.status,
@@ -153,7 +156,9 @@ export interface ExtensionUpdateCheckResult {
 
 export async function checkForAllExtensionUpdates(
   extensions: GeminiCLIExtension[],
+  extensionEnablementManager: ExtensionEnablementManager,
   dispatch: (action: ExtensionUpdateAction) => void,
+  cwd: string = process.cwd(),
 ): Promise<void> {
   dispatch({ type: 'BATCH_CHECK_START' });
   const promises: Array<Promise<void>> = [];
@@ -168,13 +173,21 @@ export async function checkForAllExtensionUpdates(
       });
       continue;
     }
+    dispatch({
+      type: 'SET_STATE',
+      payload: {
+        name: extension.name,
+        state: ExtensionUpdateState.CHECKING_FOR_UPDATES,
+      },
+    });
     promises.push(
-      checkForExtensionUpdate(extension, (updatedState) => {
-        dispatch({
-          type: 'SET_STATE',
-          payload: { name: extension.name, state: updatedState },
-        });
-      }),
+      checkForExtensionUpdate(extension, extensionEnablementManager, cwd).then(
+        (state) =>
+          dispatch({
+            type: 'SET_STATE',
+            payload: { name: extension.name, state },
+          }),
+      ),
     );
   }
   await Promise.all(promises);
