@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { SlashCommand } from '../ui/commands/types.js';
+import type { SlashCommand } from '../ui/commands/types.js';
 
 /**
  * Defines the contract for any class that can load and provide slash commands.
