@@ -14,6 +14,7 @@ import { SettingScope } from '../../config/settings.js';
 import {
   AuthType,
   clearCachedCredentialFile,
+  debugLogger,
   type Config,
 } from '@google/gemini-cli-core';
 import { useKeypress } from '../hooks/useKeypress.js';
@@ -26,7 +27,7 @@ interface AuthDialogProps {
   settings: LoadedSettings;
   setAuthState: (state: AuthState) => void;
   authError: string | null;
-  onAuthError: (error: string) => void;
+  onAuthError: (error: string | null) => void;
 }
 
 export function AuthDialog({
@@ -108,7 +109,7 @@ export function AuthDialog({
           config.isBrowserLaunchSuppressed()
         ) {
           runExitCleanup();
-          console.log(
+          debugLogger.log(
             `
 ----------------------------------------------------------------
 Logging in with Google... Please restart Gemini CLI to continue.
@@ -174,6 +175,9 @@ Logging in with Google... Please restart Gemini CLI to continue.
           items={items}
           initialIndex={initialAuthIndex}
           onSelect={handleAuthSelect}
+          onHighlight={() => {
+            onAuthError(null);
+          }}
         />
       </Box>
       {authError && (
@@ -182,7 +186,9 @@ Logging in with Google... Please restart Gemini CLI to continue.
         </Box>
       )}
       <Box marginTop={1}>
-        <Text color={theme.text.secondary}>(Use Enter to select)</Text>
+        <Text color={theme.text.secondary}>
+          (Use Enter to select, Esc to close)
+        </Text>
       </Box>
       <Box marginTop={1}>
         <Text color={theme.text.primary}>
