@@ -8,7 +8,7 @@ import { CommandKind, type SlashCommand } from './types.js';
 
 export const corgiCommand: SlashCommand = {
   name: 'corgi',
-  description: 'Toggles corgi mode.',
+  description: 'Toggles corgi mode',
   hidden: true,
   kind: CommandKind.BUILT_IN,
   action: (context, _args) => {
