@@ -4,22 +4,20 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import {
-  getErrorMessage,
-  loadServerHierarchicalMemory,
-} from '@google/gemini-cli-core';
+import { getErrorMessage } from '@google/gemini-cli-core';
 import { MessageType } from '../types.js';
+import { loadHierarchicalGeminiMemory } from '../../config/config.js';
 import type { SlashCommand, SlashCommandActionReturn } from './types.js';
 import { CommandKind } from './types.js';
 
 export const memoryCommand: SlashCommand = {
   name: 'memory',
-  description: 'Commands for interacting with memory.',
+  description: 'Commands for interacting with memory',
   kind: CommandKind.BUILT_IN,
   subCommands: [
     {
       name: 'show',
-      description: 'Show the current memory contents.',
+      description: 'Show the current memory contents',
       kind: CommandKind.BUILT_IN,
       action: async (context) => {
         const memoryContent = context.services.config?.getUserMemory() || '';
@@ -41,7 +39,7 @@ export const memoryCommand: SlashCommand = {
     },
     {
       name: 'add',
-      description: 'Add content to the memory.',
+      description: 'Add content to the memory',
       kind: CommandKind.BUILT_IN,
       action: (context, args): SlashCommandActionReturn | void => {
         if (!args || args.trim() === '') {
@@ -69,7 +67,7 @@ export const memoryCommand: SlashCommand = {
     },
     {
       name: 'refresh',
-      description: 'Refresh the memory from the source.',
+      description: 'Refresh the memory from the source',
       kind: CommandKind.BUILT_IN,
       action: async (context) => {
         context.ui.addItem(
@@ -82,24 +80,26 @@ export const memoryCommand: SlashCommand = {
 
         try {
           const config = await context.services.config;
+          const settings = context.services.settings;
           if (config) {
-            const { memoryContent, fileCount } =
-              await loadServerHierarchicalMemory(
+            const { memoryContent, fileCount, filePaths } =
+              await loadHierarchicalGeminiMemory(
                 config.getWorkingDir(),
                 config.shouldLoadMemoryFromIncludeDirectories()
                   ? config.getWorkspaceContext().getDirectories()
                   : [],
                 config.getDebugMode(),
                 config.getFileService(),
-                config.getExtensionContextFilePaths(),
-                config.getFolderTrust(),
-                context.services.settings.merged.context?.importFormat ||
-                  'tree', // Use setting or default to 'tree'
+                settings.merged,
+                config.getExtensions(),
+                config.isTrustedFolder(),
+                settings.merged.context?.importFormat || 'tree',
                 config.getFileFilteringOptions(),
-                context.services.settings.merged.context?.discoveryMaxDirs,
               );
             config.setUserMemory(memoryContent);
             config.setGeminiMdFileCount(fileCount);
+            config.setGeminiMdFilePaths(filePaths);
+            context.ui.setGeminiMdFileCount(fileCount);
 
             const successMessage =
               memoryContent.length > 0
@@ -126,5 +126,27 @@ export const memoryCommand: SlashCommand = {
         }
       },
     },
+    {
+      name: 'list',
+      description: 'Lists the paths of the GEMINI.md files in use',
+      kind: CommandKind.BUILT_IN,
+      action: async (context) => {
+        const filePaths = context.services.config?.getGeminiMdFilePaths() || [];
+        const fileCount = filePaths.length;
+
+        const messageContent =
+          fileCount > 0
+            ? `There are ${fileCount} GEMINI.md file(s) in use:\n\n${filePaths.join('\n')}`
+            : 'No GEMINI.md files in use.';
+
+        context.ui.addItem(
+          {
+            type: MessageType.INFO,
+            text: messageContent,
+          },
+          Date.now(),
+        );
+      },
+    },
   ],
 };
