@@ -4,7 +4,8 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { MessageActionReturn, SlashCommand, CommandKind } from './types.js';
+import type { MessageActionReturn, SlashCommand } from './types.js';
+import { CommandKind } from './types.js';
 import { terminalSetup } from '../utils/terminalSetup.js';
 
 /**
