@@ -9,7 +9,7 @@ import { CommandKind } from './types.js';
 
 export const themeCommand: SlashCommand = {
   name: 'theme',
-  description: 'change the theme',
+  description: 'Change the theme',
   kind: CommandKind.BUILT_IN,
   action: (_context, _args): OpenDialogActionReturn => ({
     type: 'dialog',
