@@ -20,20 +20,11 @@ export const UserMessage: React.FC<UserMessageProps> = ({ text }) => {
   const isSlashCommand = checkIsSlashCommand(text);
 
   const textColor = isSlashCommand ? theme.text.accent : theme.text.secondary;
-  const borderColor = isSlashCommand ? theme.text.accent : theme.text.secondary;
 
   return (
-    <Box
-      borderStyle="round"
-      borderColor={borderColor}
-      flexDirection="row"
-      paddingX={2}
-      paddingY={0}
-      marginY={1}
-      alignSelf="flex-start"
-    >
+    <Box flexDirection="row" paddingY={0} marginY={1} alignSelf="flex-start">
       <Box width={prefixWidth}>
-        <Text color={textColor} aria-label={SCREEN_READER_USER_PREFIX}>
+        <Text color={theme.text.accent} aria-label={SCREEN_READER_USER_PREFIX}>
           {prefix}
         </Text>
       </Box>
