@@ -15,3 +15,15 @@ export const UI_WIDTH =
 export const STREAM_DEBOUNCE_MS = 100;
 
 export const SHELL_COMMAND_NAME = 'Shell Command';
+
+export const SHELL_NAME = 'Shell';
+
+// Tool status symbols used in ToolMessage component
+export const TOOL_STATUS = {
+  SUCCESS: '✓',
+  PENDING: 'o',
+  EXECUTING: '⊷',
+  CONFIRMING: '?',
+  CANCELED: '-',
+  ERROR: 'x',
+} as const;
