@@ -6,18 +6,16 @@
 
 import { useEffect, useState } from 'react';
 
-const TERMINAL_PADDING_X = 8;
-
 export function useTerminalSize(): { columns: number; rows: number } {
   const [size, setSize] = useState({
-    columns: (process.stdout.columns || 60) - TERMINAL_PADDING_X,
+    columns: process.stdout.columns || 60,
     rows: process.stdout.rows || 20,
   });
 
   useEffect(() => {
     function updateSize() {
       setSize({
-        columns: (process.stdout.columns || 60) - TERMINAL_PADDING_X,
+        columns: process.stdout.columns || 60,
         rows: process.stdout.rows || 20,
       });
     }
