@@ -70,6 +70,9 @@ const mockConfig = {
   getUseModelRouter: () => false,
   getGeminiClient: () => null, // No client needed for these tests
   getShellExecutionConfig: () => ({ terminalWidth: 80, terminalHeight: 24 }),
+  getEnableMessageBusIntegration: () => false,
+  getMessageBus: () => null,
+  getPolicyEngine: () => null,
 } as unknown as Config;
 
 const mockTool = new MockTool({
