@@ -51,10 +51,10 @@ describe('keyMatchers', () => {
       key.ctrl && (key.name === 'x' || key.sequence === '\x18'),
     [Command.PASTE_CLIPBOARD_IMAGE]: (key: Key) => key.ctrl && key.name === 'v',
     [Command.SHOW_ERROR_DETAILS]: (key: Key) => key.ctrl && key.name === 'o',
-    [Command.TOGGLE_TOOL_DESCRIPTIONS]: (key: Key) =>
-      key.ctrl && key.name === 't',
+    [Command.SHOW_FULL_TODOS]: (key: Key) => key.ctrl && key.name === 't',
     [Command.TOGGLE_IDE_CONTEXT_DETAIL]: (key: Key) =>
       key.ctrl && key.name === 'g',
+    [Command.TOGGLE_MARKDOWN]: (key: Key) => key.meta && key.name === 'm',
     [Command.QUIT]: (key: Key) => key.ctrl && key.name === 'c',
     [Command.EXIT]: (key: Key) => key.ctrl && key.name === 'd',
     [Command.SHOW_MORE_LINES]: (key: Key) => key.ctrl && key.name === 's',
@@ -216,15 +216,20 @@ describe('keyMatchers', () => {
       negative: [createKey('o'), createKey('e', { ctrl: true })],
     },
     {
-      command: Command.TOGGLE_TOOL_DESCRIPTIONS,
+      command: Command.SHOW_FULL_TODOS,
       positive: [createKey('t', { ctrl: true })],
-      negative: [createKey('t'), createKey('s', { ctrl: true })],
+      negative: [createKey('t'), createKey('e', { ctrl: true })],
     },
     {
       command: Command.TOGGLE_IDE_CONTEXT_DETAIL,
       positive: [createKey('g', { ctrl: true })],
       negative: [createKey('g'), createKey('t', { ctrl: true })],
     },
+    {
+      command: Command.TOGGLE_MARKDOWN,
+      positive: [createKey('m', { meta: true })],
+      negative: [createKey('m'), createKey('m', { shift: true })],
+    },
     {
       command: Command.QUIT,
       positive: [createKey('c', { ctrl: true })],
