@@ -4,10 +4,13 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { isGitRepository } from '@google/gemini-cli-core';
+import { debugLogger, isGitRepository } from '@google/gemini-cli-core';
 import * as fs from 'node:fs';
 import * as path from 'node:path';
 import * as childProcess from 'node:child_process';
+import process from 'node:process';
+
+export const isDevelopment = process.env['NODE_ENV'] === 'development';
 
 export enum PackageManager {
   NPM = 'npm',
@@ -171,7 +174,7 @@ export function getInstallationInfo(
         : 'Installed with npm. Attempting to automatically update now...',
     };
   } catch (error) {
-    console.log(error);
+    debugLogger.log(error);
     return { packageManager: PackageManager.UNKNOWN, isGlobal: false };
   }
 }
