@@ -4,20 +4,39 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
+import {
+  describe,
+  it,
+  expect,
+  vi,
+  beforeEach,
+  afterEach,
+  type MockInstance,
+  type Mock,
+} from 'vitest';
 import { validateNonInteractiveAuth } from './validateNonInterActiveAuth.js';
-import { AuthType, OutputFormat } from '@google/gemini-cli-core';
+import {
+  AuthType,
+  OutputFormat,
+  makeFakeConfig,
+} from '@google/gemini-cli-core';
 import type { Config } from '@google/gemini-cli-core';
 import * as auth from './config/auth.js';
 import { type LoadedSettings } from './config/settings.js';
 
+function createLocalMockConfig(overrides: Partial<Config> = {}): Config {
+  const config = makeFakeConfig();
+  Object.assign(config, overrides);
+  return config;
+}
+
 describe('validateNonInterActiveAuth', () => {
   let originalEnvGeminiApiKey: string | undefined;
   let originalEnvVertexAi: string | undefined;
   let originalEnvGcp: string | undefined;
   let consoleErrorSpy: ReturnType<typeof vi.spyOn>;
-  let processExitSpy: ReturnType<typeof vi.spyOn>;
-  let refreshAuthMock: vi.Mock;
+  let processExitSpy: MockInstance;
+  let refreshAuthMock: Mock;
   let mockSettings: LoadedSettings;
 
   beforeEach(() => {
@@ -28,10 +47,13 @@ describe('validateNonInterActiveAuth', () => {
     delete process.env['GOOGLE_GENAI_USE_VERTEXAI'];
     delete process.env['GOOGLE_GENAI_USE_GCA'];
     consoleErrorSpy = vi.spyOn(console, 'error').mockImplementation(() => {});
-    processExitSpy = vi.spyOn(process, 'exit').mockImplementation((code) => {
-      throw new Error(`process.exit(${code}) called`);
-    });
-    refreshAuthMock = vi.fn().mockResolvedValue('refreshed');
+    processExitSpy = vi
+      .spyOn(process, 'exit')
+      .mockImplementation((code?: string | number | null | undefined) => {
+        throw new Error(`process.exit(${code}) called`);
+      });
+    vi.spyOn(auth, 'validateAuthMethod').mockReturnValue(null);
+    refreshAuthMock = vi.fn().mockImplementation(async () => 'refreshed');
     mockSettings = {
       system: { path: '', settings: {} },
       systemDefaults: { path: '', settings: {} },
@@ -73,13 +95,13 @@ describe('validateNonInterActiveAuth', () => {
   });
 
   it('exits if no auth type is configured or env vars set', async () => {
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
       getOutputFormat: vi.fn().mockReturnValue(OutputFormat.TEXT),
       getContentGeneratorConfig: vi
         .fn()
         .mockReturnValue({ authType: undefined }),
-    };
+    });
     try {
       await validateNonInteractiveAuth(
         undefined,
@@ -99,9 +121,9 @@ describe('validateNonInterActiveAuth', () => {
 
   it('uses LOGIN_WITH_GOOGLE if GOOGLE_GENAI_USE_GCA is set', async () => {
     process.env['GOOGLE_GENAI_USE_GCA'] = 'true';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -113,9 +135,9 @@ describe('validateNonInterActiveAuth', () => {
 
   it('uses USE_GEMINI if GEMINI_API_KEY is set', async () => {
     process.env['GEMINI_API_KEY'] = 'fake-key';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -129,9 +151,9 @@ describe('validateNonInterActiveAuth', () => {
     process.env['GOOGLE_GENAI_USE_VERTEXAI'] = 'true';
     process.env['GOOGLE_CLOUD_PROJECT'] = 'test-project';
     process.env['GOOGLE_CLOUD_LOCATION'] = 'us-central1';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -144,9 +166,9 @@ describe('validateNonInterActiveAuth', () => {
   it('uses USE_VERTEX_AI if GOOGLE_GENAI_USE_VERTEXAI is true and GOOGLE_API_KEY is set', async () => {
     process.env['GOOGLE_GENAI_USE_VERTEXAI'] = 'true';
     process.env['GOOGLE_API_KEY'] = 'vertex-api-key';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -162,9 +184,9 @@ describe('validateNonInterActiveAuth', () => {
     process.env['GOOGLE_GENAI_USE_VERTEXAI'] = 'true';
     process.env['GOOGLE_CLOUD_PROJECT'] = 'test-project';
     process.env['GOOGLE_CLOUD_LOCATION'] = 'us-central1';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -179,9 +201,9 @@ describe('validateNonInterActiveAuth', () => {
     process.env['GOOGLE_GENAI_USE_VERTEXAI'] = 'true';
     process.env['GOOGLE_CLOUD_PROJECT'] = 'test-project';
     process.env['GOOGLE_CLOUD_LOCATION'] = 'us-central1';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -196,9 +218,9 @@ describe('validateNonInterActiveAuth', () => {
     process.env['GEMINI_API_KEY'] = 'fake-key';
     process.env['GOOGLE_CLOUD_PROJECT'] = 'test-project';
     process.env['GOOGLE_CLOUD_LOCATION'] = 'us-central1';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
       undefined,
       undefined,
@@ -208,31 +230,30 @@ describe('validateNonInterActiveAuth', () => {
     expect(refreshAuthMock).toHaveBeenCalledWith(AuthType.USE_GEMINI);
   });
 
-  it('uses configuredAuthType if provided', async () => {
-    // Set required env var for USE_GEMINI
+  it('uses configuredAuthType over environment variables', async () => {
     process.env['GEMINI_API_KEY'] = 'fake-key';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
-      AuthType.USE_GEMINI,
+      AuthType.LOGIN_WITH_GOOGLE,
       undefined,
       nonInteractiveConfig,
       mockSettings,
     );
-    expect(refreshAuthMock).toHaveBeenCalledWith(AuthType.USE_GEMINI);
+    expect(refreshAuthMock).toHaveBeenCalledWith(AuthType.LOGIN_WITH_GOOGLE);
   });
 
   it('exits if validateAuthMethod returns error', async () => {
     // Mock validateAuthMethod to return error
     vi.spyOn(auth, 'validateAuthMethod').mockReturnValue('Auth error!');
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
       getOutputFormat: vi.fn().mockReturnValue(OutputFormat.TEXT),
       getContentGeneratorConfig: vi
         .fn()
         .mockReturnValue({ authType: undefined }),
-    };
+    });
     try {
       await validateNonInteractiveAuth(
         AuthType.USE_GEMINI,
@@ -253,10 +274,9 @@ describe('validateNonInterActiveAuth', () => {
     const validateAuthMethodSpy = vi
       .spyOn(auth, 'validateAuthMethod')
       .mockReturnValue('Auth error!');
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
-
+    });
     // Even with an invalid auth type, it should not exit
     // because validation is skipped.
     await validateNonInteractiveAuth(
@@ -273,16 +293,14 @@ describe('validateNonInterActiveAuth', () => {
     expect(refreshAuthMock).toHaveBeenCalledWith('invalid-auth-type');
   });
 
-  it('uses enforcedAuthType if provided', async () => {
-    mockSettings.merged.security.auth.enforcedType = AuthType.USE_GEMINI;
-    mockSettings.merged.security.auth.selectedType = AuthType.USE_GEMINI;
-    // Set required env var for USE_GEMINI to ensure enforcedAuthType takes precedence
+  it('succeeds if effectiveAuthType matches enforcedAuthType', async () => {
+    mockSettings.merged.security!.auth!.enforcedType = AuthType.USE_GEMINI;
     process.env['GEMINI_API_KEY'] = 'fake-key';
-    const nonInteractiveConfig = {
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
-    };
+    });
     await validateNonInteractiveAuth(
-      AuthType.USE_GEMINI,
+      undefined,
       undefined,
       nonInteractiveConfig,
       mockSettings,
@@ -290,16 +308,13 @@ describe('validateNonInterActiveAuth', () => {
     expect(refreshAuthMock).toHaveBeenCalledWith(AuthType.USE_GEMINI);
   });
 
-  it('exits if currentAuthType does not match enforcedAuthType', async () => {
-    mockSettings.merged.security.auth.enforcedType = AuthType.LOGIN_WITH_GOOGLE;
-    process.env['GOOGLE_GENAI_USE_VERTEXAI'] = 'true';
-    const nonInteractiveConfig = {
+  it('exits if configuredAuthType does not match enforcedAuthType', async () => {
+    mockSettings.merged.security!.auth!.enforcedType =
+      AuthType.LOGIN_WITH_GOOGLE;
+    const nonInteractiveConfig = createLocalMockConfig({
       refreshAuth: refreshAuthMock,
       getOutputFormat: vi.fn().mockReturnValue(OutputFormat.TEXT),
-      getContentGeneratorConfig: vi
-        .fn()
-        .mockReturnValue({ authType: undefined }),
-    };
+    });
     try {
       await validateNonInteractiveAuth(
         AuthType.USE_GEMINI,
@@ -312,27 +327,52 @@ describe('validateNonInterActiveAuth', () => {
       expect((e as Error).message).toContain('process.exit(1) called');
     }
     expect(consoleErrorSpy).toHaveBeenCalledWith(
-      'The configured auth type is oauth-personal, but the current auth type is vertex-ai. Please re-authenticate with the correct type.',
+      "The enforced authentication type is 'oauth-personal', but the current type is 'gemini-api-key'. Please re-authenticate with the correct type.",
+    );
+    expect(processExitSpy).toHaveBeenCalledWith(1);
+  });
+
+  it('exits if auth from env var does not match enforcedAuthType', async () => {
+    mockSettings.merged.security!.auth!.enforcedType =
+      AuthType.LOGIN_WITH_GOOGLE;
+    process.env['GEMINI_API_KEY'] = 'fake-key';
+    const nonInteractiveConfig = createLocalMockConfig({
+      refreshAuth: refreshAuthMock,
+      getOutputFormat: vi.fn().mockReturnValue(OutputFormat.TEXT),
+    });
+    try {
+      await validateNonInteractiveAuth(
+        undefined,
+        undefined,
+        nonInteractiveConfig,
+        mockSettings,
+      );
+      expect.fail('Should have exited');
+    } catch (e) {
+      expect((e as Error).message).toContain('process.exit(1) called');
+    }
+    expect(consoleErrorSpy).toHaveBeenCalledWith(
+      "The enforced authentication type is 'oauth-personal', but the current type is 'gemini-api-key'. Please re-authenticate with the correct type.",
     );
     expect(processExitSpy).toHaveBeenCalledWith(1);
   });
 
   describe('JSON output mode', () => {
     it('prints JSON error when no auth is configured and exits with code 1', async () => {
-      const nonInteractiveConfig = {
+      const nonInteractiveConfig = createLocalMockConfig({
         refreshAuth: refreshAuthMock,
         getOutputFormat: vi.fn().mockReturnValue(OutputFormat.JSON),
         getContentGeneratorConfig: vi
           .fn()
           .mockReturnValue({ authType: undefined }),
-      };
+      });
 
       let thrown: Error | undefined;
       try {
         await validateNonInteractiveAuth(
           undefined,
           undefined,
-          nonInteractiveConfig as unknown as Config,
+          nonInteractiveConfig,
           mockSettings,
         );
       } catch (e) {
@@ -350,23 +390,21 @@ describe('validateNonInterActiveAuth', () => {
     });
 
     it('prints JSON error when enforced auth mismatches current auth and exits with code 1', async () => {
-      mockSettings.merged.security.auth.enforcedType = AuthType.USE_GEMINI;
-      process.env['GOOGLE_GENAI_USE_GCA'] = 'true';
-
-      const nonInteractiveConfig = {
+      mockSettings.merged.security!.auth!.enforcedType = AuthType.USE_GEMINI;
+      const nonInteractiveConfig = createLocalMockConfig({
         refreshAuth: refreshAuthMock,
         getOutputFormat: vi.fn().mockReturnValue(OutputFormat.JSON),
         getContentGeneratorConfig: vi
           .fn()
           .mockReturnValue({ authType: undefined }),
-      };
+      });
 
       let thrown: Error | undefined;
       try {
         await validateNonInteractiveAuth(
+          AuthType.LOGIN_WITH_GOOGLE,
           undefined,
-          undefined,
-          nonInteractiveConfig as unknown as Config,
+          nonInteractiveConfig,
           mockSettings,
         );
       } catch (e) {
@@ -380,7 +418,7 @@ describe('validateNonInterActiveAuth', () => {
         expect(payload.error.type).toBe('Error');
         expect(payload.error.code).toBe(1);
         expect(payload.error.message).toContain(
-          'The configured auth type is gemini-api-key, but the current auth type is oauth-personal.',
+          "The enforced authentication type is 'gemini-api-key', but the current type is 'oauth-personal'. Please re-authenticate with the correct type.",
         );
       }
     });
@@ -389,20 +427,20 @@ describe('validateNonInterActiveAuth', () => {
       vi.spyOn(auth, 'validateAuthMethod').mockReturnValue('Auth error!');
       process.env['GEMINI_API_KEY'] = 'fake-key';
 
-      const nonInteractiveConfig = {
+      const nonInteractiveConfig = createLocalMockConfig({
         refreshAuth: refreshAuthMock,
         getOutputFormat: vi.fn().mockReturnValue(OutputFormat.JSON),
         getContentGeneratorConfig: vi
           .fn()
           .mockReturnValue({ authType: undefined }),
-      };
+      });
 
       let thrown: Error | undefined;
       try {
         await validateNonInteractiveAuth(
           AuthType.USE_GEMINI,
           undefined,
-          nonInteractiveConfig as unknown as Config,
+          nonInteractiveConfig,
           mockSettings,
         );
       } catch (e) {
