@@ -10,7 +10,9 @@ import { isGitRepository } from '../utils/gitUtils.js';
 import fs from 'node:fs';
 import os from 'node:os';
 import path from 'node:path';
-import { GEMINI_CONFIG_DIR } from '../tools/memoryTool.js';
+import type { Config } from '../config/config.js';
+import { CodebaseInvestigatorAgent } from '../agents/codebase-investigator.js';
+import { GEMINI_DIR } from '../utils/paths.js';
 
 // Mock tool names if they are dynamically generated or complex
 vi.mock('../tools/ls', () => ({ LSTool: { Name: 'list_directory' } }));
@@ -27,21 +29,36 @@ vi.mock('../tools/shell', () => ({
 vi.mock('../tools/write-file', () => ({
   WriteFileTool: { Name: 'write_file' },
 }));
+vi.mock('../agents/codebase-investigator.js', () => ({
+  CodebaseInvestigatorAgent: { name: 'codebase_investigator' },
+}));
 vi.mock('../utils/gitUtils', () => ({
   isGitRepository: vi.fn(),
 }));
 vi.mock('node:fs');
 
 describe('Core System Prompt (prompts.ts)', () => {
+  let mockConfig: Config;
   beforeEach(() => {
     vi.resetAllMocks();
     vi.stubEnv('GEMINI_SYSTEM_MD', undefined);
     vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', undefined);
+    mockConfig = {
+      getToolRegistry: vi.fn().mockReturnValue({
+        getAllToolNames: vi.fn().mockReturnValue([]),
+      }),
+      getEnableShellOutputEfficiency: vi.fn().mockReturnValue(true),
+      storage: {
+        getProjectTempDir: vi.fn().mockReturnValue('/tmp/project-temp'),
+      },
+      isInteractive: vi.fn().mockReturnValue(true),
+      isInteractiveShellEnabled: vi.fn().mockReturnValue(true),
+    } as unknown as Config;
   });
 
   it('should return the base prompt when no userMemory is provided', () => {
     vi.stubEnv('SANDBOX', undefined);
-    const prompt = getCoreSystemPrompt();
+    const prompt = getCoreSystemPrompt(mockConfig, '');
     expect(prompt).not.toContain('---\n\n'); // Separator should not be present
     expect(prompt).toContain('You are an interactive CLI agent'); // Check for core content
     expect(prompt).toMatchSnapshot(); // Use snapshot for base prompt structure
@@ -49,7 +66,7 @@ describe('Core System Prompt (prompts.ts)', () => {
 
   it('should return the base prompt when userMemory is empty string', () => {
     vi.stubEnv('SANDBOX', undefined);
-    const prompt = getCoreSystemPrompt('');
+    const prompt = getCoreSystemPrompt(mockConfig, '');
     expect(prompt).not.toContain('---\n\n');
     expect(prompt).toContain('You are an interactive CLI agent');
     expect(prompt).toMatchSnapshot();
@@ -57,7 +74,7 @@ describe('Core System Prompt (prompts.ts)', () => {
 
   it('should return the base prompt when userMemory is whitespace only', () => {
     vi.stubEnv('SANDBOX', undefined);
-    const prompt = getCoreSystemPrompt('   \n  \t ');
+    const prompt = getCoreSystemPrompt(mockConfig, '   \n  \t ');
     expect(prompt).not.toContain('---\n\n');
     expect(prompt).toContain('You are an interactive CLI agent');
     expect(prompt).toMatchSnapshot();
@@ -67,7 +84,7 @@ describe('Core System Prompt (prompts.ts)', () => {
     vi.stubEnv('SANDBOX', undefined);
     const memory = 'This is custom user memory.\nBe extra polite.';
     const expectedSuffix = `\n\n---\n\n${memory}`;
-    const prompt = getCoreSystemPrompt(memory);
+    const prompt = getCoreSystemPrompt(mockConfig, memory);
 
     expect(prompt.endsWith(expectedSuffix)).toBe(true);
     expect(prompt).toContain('You are an interactive CLI agent'); // Ensure base prompt follows
@@ -76,7 +93,7 @@ describe('Core System Prompt (prompts.ts)', () => {
 
   it('should include sandbox-specific instructions when SANDBOX env var is set', () => {
     vi.stubEnv('SANDBOX', 'true'); // Generic sandbox value
-    const prompt = getCoreSystemPrompt();
+    const prompt = getCoreSystemPrompt(mockConfig);
     expect(prompt).toContain('# Sandbox');
     expect(prompt).not.toContain('# macOS Seatbelt');
     expect(prompt).not.toContain('# Outside of Sandbox');
@@ -85,7 +102,7 @@ describe('Core System Prompt (prompts.ts)', () => {
 
   it('should include seatbelt-specific instructions when SANDBOX env var is "sandbox-exec"', () => {
     vi.stubEnv('SANDBOX', 'sandbox-exec');
-    const prompt = getCoreSystemPrompt();
+    const prompt = getCoreSystemPrompt(mockConfig);
     expect(prompt).toContain('# macOS Seatbelt');
     expect(prompt).not.toContain('# Sandbox');
     expect(prompt).not.toContain('# Outside of Sandbox');
@@ -93,8 +110,8 @@ describe('Core System Prompt (prompts.ts)', () => {
   });
 
   it('should include non-sandbox instructions when SANDBOX env var is not set', () => {
-    vi.stubEnv('SANDBOX', undefined); // Ensure it's not set
-    const prompt = getCoreSystemPrompt();
+    vi.stubEnv('SANDBOX', undefined); // Ensure it\'s not set
+    const prompt = getCoreSystemPrompt(mockConfig);
     expect(prompt).toContain('# Outside of Sandbox');
     expect(prompt).not.toContain('# Sandbox');
     expect(prompt).not.toContain('# macOS Seatbelt');
@@ -104,7 +121,7 @@ describe('Core System Prompt (prompts.ts)', () => {
   it('should include git instructions when in a git repo', () => {
     vi.stubEnv('SANDBOX', undefined);
     vi.mocked(isGitRepository).mockReturnValue(true);
-    const prompt = getCoreSystemPrompt();
+    const prompt = getCoreSystemPrompt(mockConfig);
     expect(prompt).toContain('# Git Repository');
     expect(prompt).toMatchSnapshot();
   });
@@ -112,22 +129,74 @@ describe('Core System Prompt (prompts.ts)', () => {
   it('should not include git instructions when not in a git repo', () => {
     vi.stubEnv('SANDBOX', undefined);
     vi.mocked(isGitRepository).mockReturnValue(false);
-    const prompt = getCoreSystemPrompt();
+    const prompt = getCoreSystemPrompt(mockConfig);
     expect(prompt).not.toContain('# Git Repository');
     expect(prompt).toMatchSnapshot();
   });
 
+  it('should return the interactive avoidance prompt when in non-interactive mode', () => {
+    vi.stubEnv('SANDBOX', undefined);
+    mockConfig.isInteractive = vi.fn().mockReturnValue(false);
+    const prompt = getCoreSystemPrompt(mockConfig, '');
+    expect(prompt).toContain('**Interactive Commands:**'); // Check for interactive prompt
+    expect(prompt).toMatchSnapshot(); // Use snapshot for base prompt structure
+  });
+
+  describe('with CodebaseInvestigator enabled', () => {
+    beforeEach(() => {
+      mockConfig = {
+        getToolRegistry: vi.fn().mockReturnValue({
+          getAllToolNames: vi
+            .fn()
+            .mockReturnValue([CodebaseInvestigatorAgent.name]),
+        }),
+        getEnableShellOutputEfficiency: vi.fn().mockReturnValue(true),
+        storage: {
+          getProjectTempDir: vi.fn().mockReturnValue('/tmp/project-temp'),
+        },
+        isInteractive: vi.fn().mockReturnValue(false),
+        isInteractiveShellEnabled: vi.fn().mockReturnValue(false),
+      } as unknown as Config;
+    });
+
+    it('should include CodebaseInvestigator instructions in the prompt', () => {
+      const prompt = getCoreSystemPrompt(mockConfig);
+      expect(prompt).toContain(
+        `your **first and primary tool** must be '${CodebaseInvestigatorAgent.name}'`,
+      );
+      expect(prompt).toContain(
+        `do not ignore the output of '${CodebaseInvestigatorAgent.name}'`,
+      );
+      expect(prompt).not.toContain(
+        "Use 'search_file_content' and 'glob' search tools extensively",
+      );
+    });
+  });
+
+  describe('with CodebaseInvestigator disabled', () => {
+    // No beforeEach needed, will use the default from the parent describe
+    it('should include standard tool instructions in the prompt', () => {
+      const prompt = getCoreSystemPrompt(mockConfig);
+      expect(prompt).not.toContain(
+        `your **first and primary tool** must be '${CodebaseInvestigatorAgent.name}'`,
+      );
+      expect(prompt).toContain(
+        "Use 'search_file_content' and 'glob' search tools extensively",
+      );
+    });
+  });
+
   describe('GEMINI_SYSTEM_MD environment variable', () => {
     it('should use default prompt when GEMINI_SYSTEM_MD is "false"', () => {
       vi.stubEnv('GEMINI_SYSTEM_MD', 'false');
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).not.toHaveBeenCalled();
       expect(prompt).not.toContain('custom system prompt');
     });
 
     it('should use default prompt when GEMINI_SYSTEM_MD is "0"', () => {
       vi.stubEnv('GEMINI_SYSTEM_MD', '0');
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).not.toHaveBeenCalled();
       expect(prompt).not.toContain('custom system prompt');
     });
@@ -136,33 +205,29 @@ describe('Core System Prompt (prompts.ts)', () => {
       const customPath = '/non/existent/path/system.md';
       vi.stubEnv('GEMINI_SYSTEM_MD', customPath);
       vi.mocked(fs.existsSync).mockReturnValue(false);
-      expect(() => getCoreSystemPrompt()).toThrow(
+      expect(() => getCoreSystemPrompt(mockConfig)).toThrow(
         `missing system prompt file '${path.resolve(customPath)}'`,
       );
     });
 
     it('should read from default path when GEMINI_SYSTEM_MD is "true"', () => {
-      const defaultPath = path.resolve(
-        path.join(GEMINI_CONFIG_DIR, 'system.md'),
-      );
+      const defaultPath = path.resolve(path.join(GEMINI_DIR, 'system.md'));
       vi.stubEnv('GEMINI_SYSTEM_MD', 'true');
       vi.mocked(fs.existsSync).mockReturnValue(true);
       vi.mocked(fs.readFileSync).mockReturnValue('custom system prompt');
 
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).toHaveBeenCalledWith(defaultPath, 'utf8');
       expect(prompt).toBe('custom system prompt');
     });
 
     it('should read from default path when GEMINI_SYSTEM_MD is "1"', () => {
-      const defaultPath = path.resolve(
-        path.join(GEMINI_CONFIG_DIR, 'system.md'),
-      );
+      const defaultPath = path.resolve(path.join(GEMINI_DIR, 'system.md'));
       vi.stubEnv('GEMINI_SYSTEM_MD', '1');
       vi.mocked(fs.existsSync).mockReturnValue(true);
       vi.mocked(fs.readFileSync).mockReturnValue('custom system prompt');
 
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).toHaveBeenCalledWith(defaultPath, 'utf8');
       expect(prompt).toBe('custom system prompt');
     });
@@ -173,7 +238,7 @@ describe('Core System Prompt (prompts.ts)', () => {
       vi.mocked(fs.existsSync).mockReturnValue(true);
       vi.mocked(fs.readFileSync).mockReturnValue('custom system prompt');
 
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).toHaveBeenCalledWith(customPath, 'utf8');
       expect(prompt).toBe('custom system prompt');
     });
@@ -187,7 +252,7 @@ describe('Core System Prompt (prompts.ts)', () => {
       vi.mocked(fs.existsSync).mockReturnValue(true);
       vi.mocked(fs.readFileSync).mockReturnValue('custom system prompt');
 
-      const prompt = getCoreSystemPrompt();
+      const prompt = getCoreSystemPrompt(mockConfig);
       expect(fs.readFileSync).toHaveBeenCalledWith(
         path.resolve(expectedPath),
         'utf8',
@@ -199,22 +264,20 @@ describe('Core System Prompt (prompts.ts)', () => {
   describe('GEMINI_WRITE_SYSTEM_MD environment variable', () => {
     it('should not write to file when GEMINI_WRITE_SYSTEM_MD is "false"', () => {
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', 'false');
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).not.toHaveBeenCalled();
     });
 
     it('should not write to file when GEMINI_WRITE_SYSTEM_MD is "0"', () => {
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', '0');
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).not.toHaveBeenCalled();
     });
 
     it('should write to default path when GEMINI_WRITE_SYSTEM_MD is "true"', () => {
-      const defaultPath = path.resolve(
-        path.join(GEMINI_CONFIG_DIR, 'system.md'),
-      );
+      const defaultPath = path.resolve(path.join(GEMINI_DIR, 'system.md'));
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', 'true');
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).toHaveBeenCalledWith(
         defaultPath,
         expect.any(String),
@@ -222,11 +285,9 @@ describe('Core System Prompt (prompts.ts)', () => {
     });
 
     it('should write to default path when GEMINI_WRITE_SYSTEM_MD is "1"', () => {
-      const defaultPath = path.resolve(
-        path.join(GEMINI_CONFIG_DIR, 'system.md'),
-      );
+      const defaultPath = path.resolve(path.join(GEMINI_DIR, 'system.md'));
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', '1');
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).toHaveBeenCalledWith(
         defaultPath,
         expect.any(String),
@@ -236,7 +297,7 @@ describe('Core System Prompt (prompts.ts)', () => {
     it('should write to custom path when GEMINI_WRITE_SYSTEM_MD provides one', () => {
       const customPath = path.resolve('/custom/path/system.md');
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', customPath);
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).toHaveBeenCalledWith(
         customPath,
         expect.any(String),
@@ -249,7 +310,7 @@ describe('Core System Prompt (prompts.ts)', () => {
       const customPath = '~/custom/system.md';
       const expectedPath = path.join(homeDir, 'custom/system.md');
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', customPath);
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).toHaveBeenCalledWith(
         path.resolve(expectedPath),
         expect.any(String),
@@ -262,7 +323,7 @@ describe('Core System Prompt (prompts.ts)', () => {
       const customPath = '~';
       const expectedPath = homeDir;
       vi.stubEnv('GEMINI_WRITE_SYSTEM_MD', customPath);
-      getCoreSystemPrompt();
+      getCoreSystemPrompt(mockConfig);
       expect(fs.writeFileSync).toHaveBeenCalledWith(
         path.resolve(expectedPath),
         expect.any(String),
