@@ -4,8 +4,8 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { Config } from '../config/config.js';
-import { DiscoveredMCPPrompt } from '../tools/mcp-client.js';
+import type { Config } from '../config/config.js';
+import type { DiscoveredMCPPrompt } from '../tools/mcp-client.js';
 
 export function getMCPServerPrompts(
   config: Config,
