@@ -5,6 +5,7 @@
  */
 
 import type { DiscoveredMCPPrompt } from '../tools/mcp-client.js';
+import { debugLogger } from '../utils/debugLogger.js';
 
 export class PromptRegistry {
   private prompts: Map<string, DiscoveredMCPPrompt> = new Map();
@@ -16,7 +17,7 @@ export class PromptRegistry {
   registerPrompt(prompt: DiscoveredMCPPrompt): void {
     if (this.prompts.has(prompt.name)) {
       const newName = `${prompt.serverName}_${prompt.name}`;
-      console.warn(
+      debugLogger.warn(
         `Prompt with name "${prompt.name}" is already registered. Renaming to "${newName}".`,
       );
       this.prompts.set(newName, { ...prompt, name: newName });
