@@ -20,7 +20,8 @@ import {
   ApiRequestPhase,
 } from './metrics.js';
 import { makeFakeConfig } from '../test-utils/config.js';
-import { ModelRoutingEvent } from './types.js';
+import { ModelRoutingEvent, AgentFinishEvent } from './types.js';
+import { AgentTerminateMode } from '../agents/types.js';
 
 const mockCounterAddFn: Mock<
   (value: number, attributes?: Attributes, context?: Context) => void
@@ -61,10 +62,16 @@ function originalOtelMockFactory() {
       setLogger: vi.fn(),
       warn: vi.fn(),
     },
+    DiagConsoleLogger: vi.fn(),
+    DiagLogLevel: {
+      NONE: 0,
+      INFO: 1,
+    },
   } as const;
 }
 
 vi.mock('@opentelemetry/api');
+vi.mock('./telemetryAttributes.js');
 
 describe('Telemetry Metrics', () => {
   let initializeMetricsModule: typeof import('./metrics.js').initializeMetrics;
@@ -82,6 +89,10 @@ describe('Telemetry Metrics', () => {
   let recordPerformanceScoreModule: typeof import('./metrics.js').recordPerformanceScore;
   let recordPerformanceRegressionModule: typeof import('./metrics.js').recordPerformanceRegression;
   let recordBaselineComparisonModule: typeof import('./metrics.js').recordBaselineComparison;
+  let recordGenAiClientTokenUsageModule: typeof import('./metrics.js').recordGenAiClientTokenUsage;
+  let recordGenAiClientOperationDurationModule: typeof import('./metrics.js').recordGenAiClientOperationDuration;
+  let recordFlickerFrameModule: typeof import('./metrics.js').recordFlickerFrame;
+  let recordAgentRunMetricsModule: typeof import('./metrics.js').recordAgentRunMetrics;
 
   beforeEach(async () => {
     vi.resetModules();
@@ -91,6 +102,13 @@ describe('Telemetry Metrics', () => {
       return actualApi;
     });
 
+    const { getCommonAttributes } = await import('./telemetryAttributes.js');
+    (getCommonAttributes as Mock).mockReturnValue({
+      'session.id': 'test-session-id',
+      'installation.id': 'test-installation-id',
+      'user.email': 'test@example.com',
+    });
+
     const metricsJsModule = await import('./metrics.js');
     initializeMetricsModule = metricsJsModule.initializeMetrics;
     recordTokenUsageMetricsModule = metricsJsModule.recordTokenUsageMetrics;
@@ -110,6 +128,12 @@ describe('Telemetry Metrics', () => {
     recordPerformanceRegressionModule =
       metricsJsModule.recordPerformanceRegression;
     recordBaselineComparisonModule = metricsJsModule.recordBaselineComparison;
+    recordGenAiClientTokenUsageModule =
+      metricsJsModule.recordGenAiClientTokenUsage;
+    recordGenAiClientOperationDurationModule =
+      metricsJsModule.recordGenAiClientOperationDuration;
+    recordFlickerFrameModule = metricsJsModule.recordFlickerFrame;
+    recordAgentRunMetricsModule = metricsJsModule.recordAgentRunMetrics;
 
     const otelApiModule = await import('@opentelemetry/api');
 
@@ -124,6 +148,45 @@ describe('Telemetry Metrics', () => {
     mockCreateHistogramFn.mockReturnValue(mockHistogramInstance);
   });
 
+  describe('recordFlickerFrame', () => {
+    it('does not record metrics if not initialized', () => {
+      const config = makeFakeConfig({});
+      recordFlickerFrameModule(config);
+      expect(mockCounterAddFn).not.toHaveBeenCalled();
+    });
+
+    it('records a flicker frame event when initialized', () => {
+      const config = makeFakeConfig({});
+      initializeMetricsModule(config);
+      recordFlickerFrameModule(config);
+
+      // Called for session, then for flicker
+      expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
+      expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+      });
+    });
+  });
+
+  describe('initializeMetrics', () => {
+    const mockConfig = {
+      getSessionId: () => 'test-session-id',
+      getTelemetryEnabled: () => true,
+    } as unknown as Config;
+
+    it('should apply common attributes including email', () => {
+      initializeMetricsModule(mockConfig);
+
+      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+      });
+    });
+  });
+
   describe('recordChatCompressionMetrics', () => {
     it('does not record metrics if not initialized', () => {
       const lol = makeFakeConfig({});
@@ -147,6 +210,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         tokens_after: 100,
         tokens_before: 200,
       });
@@ -176,9 +241,13 @@ describe('Telemetry Metrics', () => {
       expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
       expect(mockCounterAddFn).toHaveBeenNthCalledWith(1, 1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
       });
       expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 100, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-pro',
         type: 'input',
       });
@@ -194,6 +263,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(50, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-pro',
         type: 'output',
       });
@@ -204,6 +275,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(25, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-pro',
         type: 'thought',
       });
@@ -214,6 +287,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(75, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-pro',
         type: 'cache',
       });
@@ -224,6 +299,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(125, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-pro',
         type: 'tool',
       });
@@ -239,6 +316,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(200, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         model: 'gemini-ultra',
         type: 'input',
       });
@@ -273,9 +352,13 @@ describe('Telemetry Metrics', () => {
       expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
       expect(mockCounterAddFn).toHaveBeenNthCalledWith(1, 1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
       });
       expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.CREATE,
         lines: 10,
         mimetype: 'text/plain',
@@ -292,6 +375,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.READ,
       });
     });
@@ -306,6 +391,8 @@ describe('Telemetry Metrics', () => {
       });
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.UPDATE,
         mimetype: 'application/javascript',
       });
@@ -321,6 +408,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.UPDATE,
       });
     });
@@ -338,6 +427,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.UPDATE,
         lines: 10,
         mimetype: 'text/plain',
@@ -355,6 +446,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         operation: FileOperation.UPDATE,
       });
     });
@@ -394,6 +487,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockHistogramRecordFn).toHaveBeenCalledWith(150, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         'routing.decision_model': 'gemini-pro',
         'routing.decision_source': 'default',
       });
@@ -415,6 +510,8 @@ describe('Telemetry Metrics', () => {
 
       expect(mockHistogramRecordFn).toHaveBeenCalledWith(200, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         'routing.decision_model': 'gemini-pro',
         'routing.decision_source': 'classifier',
       });
@@ -422,12 +519,264 @@ describe('Telemetry Metrics', () => {
       expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
       expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
         'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
         'routing.decision_source': 'classifier',
         'routing.error_message': 'test-error',
       });
     });
   });
 
+  describe('recordAgentRunMetrics', () => {
+    const mockConfig = {
+      getSessionId: () => 'test-session-id',
+      getTelemetryEnabled: () => true,
+    } as unknown as Config;
+
+    it('should not record metrics if not initialized', () => {
+      const event = new AgentFinishEvent(
+        'agent-123',
+        'TestAgent',
+        1000,
+        5,
+        AgentTerminateMode.GOAL,
+      );
+      recordAgentRunMetricsModule(mockConfig, event);
+      expect(mockCounterAddFn).not.toHaveBeenCalled();
+      expect(mockHistogramRecordFn).not.toHaveBeenCalled();
+    });
+
+    it('should record agent run metrics', () => {
+      initializeMetricsModule(mockConfig);
+      mockCounterAddFn.mockClear();
+      mockHistogramRecordFn.mockClear();
+
+      const event = new AgentFinishEvent(
+        'agent-123',
+        'TestAgent',
+        1000,
+        5,
+        AgentTerminateMode.GOAL,
+      );
+      recordAgentRunMetricsModule(mockConfig, event);
+
+      // Verify agent run counter
+      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+        agent_name: 'TestAgent',
+        terminate_reason: 'GOAL',
+      });
+
+      // Verify agent duration histogram
+      expect(mockHistogramRecordFn).toHaveBeenCalledWith(1000, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+        agent_name: 'TestAgent',
+      });
+
+      // Verify agent turns histogram
+      expect(mockHistogramRecordFn).toHaveBeenCalledWith(5, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+        agent_name: 'TestAgent',
+      });
+    });
+  });
+
+  describe('OpenTelemetry GenAI Semantic Convention Metrics', () => {
+    const mockConfig = {
+      getSessionId: () => 'test-session-id',
+      getTelemetryEnabled: () => true,
+    } as unknown as Config;
+
+    describe('recordGenAiClientTokenUsage', () => {
+      it('should not record metrics when not initialized', () => {
+        recordGenAiClientTokenUsageModule(mockConfig, 100, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+          'gen_ai.token.type': 'input',
+        });
+
+        expect(mockHistogramRecordFn).not.toHaveBeenCalled();
+      });
+
+      it('should record input token usage with correct attributes', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientTokenUsageModule(mockConfig, 150, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+          'gen_ai.token.type': 'input',
+          'gen_ai.request.model': 'gemini-2.0-flash',
+          'gen_ai.response.model': 'gemini-2.0-flash',
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(150, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+          'gen_ai.token.type': 'input',
+          'gen_ai.request.model': 'gemini-2.0-flash',
+          'gen_ai.response.model': 'gemini-2.0-flash',
+        });
+      });
+
+      it('should record output token usage with correct attributes', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientTokenUsageModule(mockConfig, 75, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.token.type': 'output',
+          'gen_ai.request.model': 'gemini-pro',
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(75, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.token.type': 'output',
+          'gen_ai.request.model': 'gemini-pro',
+        });
+      });
+
+      it('should record token usage with optional attributes', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientTokenUsageModule(mockConfig, 200, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.token.type': 'input',
+          'gen_ai.request.model': 'text-embedding-004',
+          'server.address': 'aiplatform.googleapis.com',
+          'server.port': 443,
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(200, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.token.type': 'input',
+          'gen_ai.request.model': 'text-embedding-004',
+          'server.address': 'aiplatform.googleapis.com',
+          'server.port': 443,
+        });
+      });
+    });
+
+    describe('recordGenAiClientOperationDuration', () => {
+      it('should not record metrics when not initialized', () => {
+        recordGenAiClientOperationDurationModule(mockConfig, 2.5, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+        });
+
+        expect(mockHistogramRecordFn).not.toHaveBeenCalled();
+      });
+
+      it('should record successful operation duration with correct attributes', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientOperationDurationModule(mockConfig, 1.25, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+          'gen_ai.request.model': 'gemini-2.0-flash',
+          'gen_ai.response.model': 'gemini-2.0-flash',
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(1.25, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+          'gen_ai.request.model': 'gemini-2.0-flash',
+          'gen_ai.response.model': 'gemini-2.0-flash',
+        });
+      });
+
+      it('should record failed operation duration with error type', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientOperationDurationModule(mockConfig, 3.75, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.request.model': 'gemini-pro',
+          'error.type': 'quota_exceeded',
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(3.75, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.request.model': 'gemini-pro',
+          'error.type': 'quota_exceeded',
+        });
+      });
+
+      it('should record operation duration with server details', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientOperationDurationModule(mockConfig, 0.95, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.request.model': 'gemini-1.5-pro',
+          'gen_ai.response.model': 'gemini-1.5-pro-001',
+          'server.address': 'us-central1-aiplatform.googleapis.com',
+          'server.port': 443,
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(0.95, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.vertex_ai',
+          'gen_ai.request.model': 'gemini-1.5-pro',
+          'gen_ai.response.model': 'gemini-1.5-pro-001',
+          'server.address': 'us-central1-aiplatform.googleapis.com',
+          'server.port': 443,
+        });
+      });
+
+      it('should handle minimal required attributes', () => {
+        initializeMetricsModule(mockConfig);
+        mockHistogramRecordFn.mockClear();
+
+        recordGenAiClientOperationDurationModule(mockConfig, 2.1, {
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+        });
+
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(2.1, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          'gen_ai.operation.name': 'generate_content',
+          'gen_ai.provider.name': 'gcp.gen_ai',
+        });
+      });
+    });
+  });
+
   describe('Performance Monitoring Metrics', () => {
     const mockConfig = {
       getSessionId: () => 'test-session-id',
@@ -470,6 +819,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(150, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           phase: 'settings_loading',
           auth_type: 'gemini',
           telemetry_enabled: true,
@@ -485,6 +836,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(50, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           phase: 'cleanup',
         });
       });
@@ -507,6 +860,8 @@ describe('Telemetry Metrics', () => {
           floatingPointDuration,
           {
             'session.id': 'test-session-id',
+            'installation.id': 'test-installation-id',
+            'user.email': 'test@example.com',
             phase: 'total_startup',
             is_tty: true,
             has_question: false,
@@ -527,6 +882,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(15728640, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           memory_type: 'heap_used',
           component: 'startup',
         });
@@ -552,16 +909,22 @@ describe('Telemetry Metrics', () => {
         expect(mockHistogramRecordFn).toHaveBeenCalledTimes(3); // One for each call
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(1, 31457280, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           memory_type: 'heap_total',
           component: 'api_call',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(2, 2097152, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           memory_type: 'external',
           component: 'tool_execution',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(3, 41943040, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           memory_type: 'rss',
           component: 'memory_monitor',
         });
@@ -577,6 +940,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(15728640, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           memory_type: 'heap_used',
         });
       });
@@ -593,6 +958,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(85.5, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           component: 'tool_execution',
         });
       });
@@ -605,6 +972,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(42.3, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
         });
       });
     });
@@ -618,6 +987,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(3, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
         });
       });
 
@@ -629,6 +1000,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(0, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
         });
       });
     });
@@ -645,6 +1018,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(25, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           function_name: 'Read',
           phase: 'validation',
         });
@@ -670,16 +1045,22 @@ describe('Telemetry Metrics', () => {
         expect(mockHistogramRecordFn).toHaveBeenCalledTimes(3); // One for each call
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(1, 50, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           function_name: 'Bash',
           phase: 'preparation',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(2, 1500, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           function_name: 'Bash',
           phase: 'execution',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(3, 75, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           function_name: 'Bash',
           phase: 'result_processing',
         });
@@ -699,6 +1080,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(0.85, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           metric: 'cache_hit_rate',
           context: 'api_request',
@@ -716,6 +1099,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(125.5, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           metric: 'tokens_per_operation',
         });
@@ -734,6 +1119,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(15, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           phase: 'request_preparation',
         });
@@ -759,16 +1146,22 @@ describe('Telemetry Metrics', () => {
         expect(mockHistogramRecordFn).toHaveBeenCalledTimes(3); // One for each call
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(1, 250, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           phase: 'network_latency',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(2, 100, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           phase: 'response_processing',
         });
         expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(3, 50, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           model: 'gemini-pro',
           phase: 'token_processing',
         });
@@ -787,6 +1180,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(85.5, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           category: 'memory_efficiency',
           baseline: 80.0,
         });
@@ -802,6 +1197,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(92.3, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           category: 'overall_performance',
         });
       });
@@ -823,6 +1220,8 @@ describe('Telemetry Metrics', () => {
         // Verify regression counter
         expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'startup_time',
           severity: 'medium',
           current_value: 1200,
@@ -832,6 +1231,8 @@ describe('Telemetry Metrics', () => {
         // Verify baseline comparison histogram (20% increase)
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(20, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'startup_time',
           severity: 'medium',
           current_value: 1200,
@@ -854,6 +1255,8 @@ describe('Telemetry Metrics', () => {
         // Verify regression counter still recorded
         expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'memory_usage',
           severity: 'high',
           current_value: 100,
@@ -883,6 +1286,8 @@ describe('Telemetry Metrics', () => {
 
         expect(mockCounterAddFn).toHaveBeenNthCalledWith(1, 1, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'api_latency',
           severity: 'low',
           current_value: 500,
@@ -890,6 +1295,8 @@ describe('Telemetry Metrics', () => {
         });
         expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'cpu_usage',
           severity: 'high',
           current_value: 90,
@@ -913,6 +1320,8 @@ describe('Telemetry Metrics', () => {
         // 20% increase: (120 - 100) / 100 * 100 = 20%
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(20, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'memory_usage',
           category: 'performance_tracking',
           current_value: 120,
@@ -934,6 +1343,8 @@ describe('Telemetry Metrics', () => {
         // 20% decrease: (800 - 1000) / 1000 * 100 = -20%
         expect(mockHistogramRecordFn).toHaveBeenCalledWith(-20, {
           'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
           metric: 'startup_time',
           category: 'optimization',
           current_value: 800,
