@@ -4,17 +4,23 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { describe, it, expect, beforeEach, afterEach } from 'vitest';
-import { ReadFileTool, ReadFileToolParams } from './read-file.js';
+import { describe, it, expect, beforeEach, afterEach, vi } from 'vitest';
+import type { ReadFileToolParams } from './read-file.js';
+import { ReadFileTool } from './read-file.js';
 import { ToolErrorType } from './tool-error.js';
-import path from 'path';
-import os from 'os';
-import fs from 'fs';
-import fsp from 'fs/promises';
-import { Config } from '../config/config.js';
+import path from 'node:path';
+import os from 'node:os';
+import fs from 'node:fs';
+import fsp from 'node:fs/promises';
+import type { Config } from '../config/config.js';
 import { FileDiscoveryService } from '../services/fileDiscoveryService.js';
+import { StandardFileSystemService } from '../services/fileSystemService.js';
 import { createMockWorkspaceContext } from '../test-utils/mockWorkspaceContext.js';
-import { ToolInvocation, ToolResult } from './tools.js';
+import type { ToolInvocation, ToolResult } from './tools.js';
+
+vi.mock('../telemetry/loggers.js', () => ({
+  logFileOperation: vi.fn(),
+}));
 
 describe('ReadFileTool', () => {
   let tempRootDir: string;
@@ -29,8 +35,12 @@ describe('ReadFileTool', () => {
 
     const mockConfigInstance = {
       getFileService: () => new FileDiscoveryService(tempRootDir),
+      getFileSystemService: () => new StandardFileSystemService(),
       getTargetDir: () => tempRootDir,
       getWorkspaceContext: () => createMockWorkspaceContext(tempRootDir),
+      storage: {
+        getProjectTempDir: () => path.join(tempRootDir, '.temp'),
+      },
     } as unknown as Config;
     tool = new ReadFileTool(mockConfigInstance);
   });
@@ -69,6 +79,33 @@ describe('ReadFileTool', () => {
       );
     });
 
+    it('should allow access to files in project temp directory', () => {
+      const tempDir = path.join(tempRootDir, '.temp');
+      const params: ReadFileToolParams = {
+        absolute_path: path.join(tempDir, 'temp-file.txt'),
+      };
+      const result = tool.build(params);
+      expect(typeof result).not.toBe('string');
+    });
+
+    it('should show temp directory in error message when path is outside workspace and temp dir', () => {
+      const params: ReadFileToolParams = {
+        absolute_path: '/completely/outside/path.txt',
+      };
+      expect(() => tool.build(params)).toThrow(
+        /File path must be within one of the workspace directories.*or within the project temp directory/,
+      );
+    });
+
+    it('should throw error if path is empty', () => {
+      const params: ReadFileToolParams = {
+        absolute_path: '',
+      };
+      expect(() => tool.build(params)).toThrow(
+        /The 'absolute_path' parameter must be non-empty./,
+      );
+    });
+
     it('should throw error if offset is negative', () => {
       const params: ReadFileToolParams = {
         absolute_path: path.join(tempRootDir, 'test.txt'),
@@ -208,7 +245,7 @@ describe('ReadFileTool', () => {
         returnDisplay: 'Path is a directory.',
         error: {
           message: `Path is a directory, not a file: ${dirPath}`,
-          type: ToolErrorType.INVALID_TOOL_PARAMS,
+          type: ToolErrorType.TARGET_IS_DIRECTORY,
         },
       });
     });
@@ -393,6 +430,24 @@ describe('ReadFileTool', () => {
       );
     });
 
+    it('should successfully read files from project temp directory', async () => {
+      const tempDir = path.join(tempRootDir, '.temp');
+      await fsp.mkdir(tempDir, { recursive: true });
+      const tempFilePath = path.join(tempDir, 'temp-output.txt');
+      const tempFileContent = 'This is temporary output content';
+      await fsp.writeFile(tempFilePath, tempFileContent, 'utf-8');
+
+      const params: ReadFileToolParams = { absolute_path: tempFilePath };
+      const invocation = tool.build(params) as ToolInvocation<
+        ReadFileToolParams,
+        ToolResult
+      >;
+
+      const result = await invocation.execute(abortSignal);
+      expect(result.llmContent).toBe(tempFileContent);
+      expect(result.returnDisplay).toBe('');
+    });
+
     describe('with .geminiignore', () => {
       beforeEach(async () => {
         await fsp.writeFile(
