@@ -6,6 +6,7 @@
 
 import * as fs from 'node:fs';
 import * as path from 'node:path';
+import * as crypto from 'node:crypto';
 import * as Diff from 'diff';
 import {
   BaseDeclarativeTool,
@@ -23,7 +24,6 @@ import { makeRelative, shortenPath } from '../utils/paths.js';
 import { isNodeError } from '../utils/errors.js';
 import { type Config, ApprovalMode } from '../config/config.js';
 import { DEFAULT_DIFF_OPTIONS, getDiffStat } from './diffOptions.js';
-import { ReadFileTool } from './read-file.js';
 import {
   type ModifiableDeclarativeTool,
   type ModifyContext,
@@ -32,7 +32,14 @@ import { IdeClient } from '../ide/ide-client.js';
 import { FixLLMEditWithInstruction } from '../utils/llm-edit-fixer.js';
 import { applyReplacement } from './edit.js';
 import { safeLiteralReplace } from '../utils/textUtils.js';
+import { SmartEditStrategyEvent } from '../telemetry/types.js';
+import { logSmartEditStrategy } from '../telemetry/loggers.js';
+import { SmartEditCorrectionEvent } from '../telemetry/types.js';
+import { logSmartEditCorrectionEvent } from '../telemetry/loggers.js';
 
+import { correctPath } from '../utils/pathCorrector.js';
+import { EDIT_TOOL_NAME, READ_FILE_TOOL_NAME } from './tool-names.js';
+import { debugLogger } from '../utils/debugLogger.js';
 interface ReplacementContext {
   params: EditToolParams;
   currentContent: string;
@@ -46,6 +53,15 @@ interface ReplacementResult {
   finalNewString: string;
 }
 
+/**
+ * Creates a SHA256 hash of the given content.
+ * @param content The string content to hash.
+ * @returns A hex-encoded hash string.
+ */
+function hashContent(content: string): string {
+  return crypto.createHash('sha256').update(content).digest('hex');
+}
+
 function restoreTrailingNewline(
   originalContent: string,
   modifiedContent: string,
@@ -229,6 +245,7 @@ function detectLineEnding(content: string): '\r\n' | '\n' {
 }
 
 export async function calculateReplacement(
+  config: Config,
   context: ReplacementContext,
 ): Promise<ReplacementResult> {
   const { currentContent, params } = context;
@@ -247,16 +264,22 @@ export async function calculateReplacement(
 
   const exactResult = await calculateExactReplacement(context);
   if (exactResult) {
+    const event = new SmartEditStrategyEvent('exact');
+    logSmartEditStrategy(config, event);
     return exactResult;
   }
 
   const flexibleResult = await calculateFlexibleReplacement(context);
   if (flexibleResult) {
+    const event = new SmartEditStrategyEvent('flexible');
+    logSmartEditStrategy(config, event);
     return flexibleResult;
   }
 
   const regexResult = await calculateRegexReplacement(context);
   if (regexResult) {
+    const event = new SmartEditStrategyEvent('regex');
+    logSmartEditStrategy(config, event);
     return regexResult;
   }
 
@@ -280,7 +303,7 @@ export function getErrorReplaceResult(
   if (occurrences === 0) {
     error = {
       display: `Failed to edit, could not find the string to replace.`,
-      raw: `Failed to edit, 0 occurrences found for old_string (${finalOldString}). Original old_string was (${params.old_string}) in ${params.file_path}. No edits made. The exact text in old_string was not found. Ensure you're not escaping content incorrectly and check whitespace, indentation, and context. Use ${ReadFileTool.Name} tool to verify.`,
+      raw: `Failed to edit, 0 occurrences found for old_string (${finalOldString}). Original old_string was (${params.old_string}) in ${params.file_path}. No edits made. The exact text in old_string was not found. Ensure you're not escaping content incorrectly and check whitespace, indentation, and context. Use ${READ_FILE_TOOL_NAME} tool to verify.`,
       type: ToolErrorType.EDIT_NO_OCCURRENCE_FOUND,
     };
   } else if (occurrences !== expectedReplacements) {
@@ -363,12 +386,30 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
     abortSignal: AbortSignal,
     originalLineEnding: '\r\n' | '\n',
   ): Promise<CalculatedEdit> {
+    // In order to keep from clobbering edits made outside our system,
+    // check if the file has been modified since we first read it.
+    let errorForLlmEditFixer = initialError.raw;
+    let contentForLlmEditFixer = currentContent;
+
+    const initialContentHash = hashContent(currentContent);
+    const onDiskContent = await this.config
+      .getFileSystemService()
+      .readTextFile(params.file_path);
+    const onDiskContentHash = hashContent(onDiskContent.replace(/\r\n/g, '\n'));
+
+    if (initialContentHash !== onDiskContentHash) {
+      // The file has changed on disk since we first read it.
+      // Use the latest content for the correction attempt.
+      contentForLlmEditFixer = onDiskContent.replace(/\r\n/g, '\n');
+      errorForLlmEditFixer = `The initial edit attempt failed with the following error: "${initialError.raw}". However, the file has been modified by either the user or an external process since that edit attempt. The file content provided to you is the latest version. Please base your correction on this new content.`;
+    }
+
     const fixedEdit = await FixLLMEditWithInstruction(
       params.instruction,
       params.old_string,
       params.new_string,
-      initialError.raw,
-      currentContent,
+      errorForLlmEditFixer,
+      contentForLlmEditFixer,
       this.config.getBaseLlmClient(),
       abortSignal,
     );
@@ -388,13 +429,13 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
       };
     }
 
-    const secondAttemptResult = await calculateReplacement({
+    const secondAttemptResult = await calculateReplacement(this.config, {
       params: {
         ...params,
         old_string: fixedEdit.search,
         new_string: fixedEdit.replace,
       },
-      currentContent,
+      currentContent: contentForLlmEditFixer,
       abortSignal,
     });
 
@@ -407,9 +448,12 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
     );
 
     if (secondError) {
-      // The fix failed, return the original error
+      // The fix failed, log failure and return the original error
+      const event = new SmartEditCorrectionEvent('failure');
+      logSmartEditCorrectionEvent(this.config, event);
+
       return {
-        currentContent,
+        currentContent: contentForLlmEditFixer,
         newContent: currentContent,
         occurrences: 0,
         isNewFile: false,
@@ -418,8 +462,11 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
       };
     }
 
+    const event = new SmartEditCorrectionEvent('success');
+    logSmartEditCorrectionEvent(this.config, event);
+
     return {
-      currentContent,
+      currentContent: contentForLlmEditFixer,
       newContent: secondAttemptResult.newContent,
       occurrences: secondAttemptResult.occurrences,
       isNewFile: false,
@@ -516,7 +563,7 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
       };
     }
 
-    const replacementResult = await calculateReplacement({
+    const replacementResult = await calculateReplacement(this.config, {
       params,
       currentContent,
       abortSignal,
@@ -570,12 +617,12 @@ class EditToolInvocation implements ToolInvocation<EditToolParams, ToolResult> {
         throw error;
       }
       const errorMsg = error instanceof Error ? error.message : String(error);
-      console.log(`Error preparing edit: ${errorMsg}`);
+      debugLogger.log(`Error preparing edit: ${errorMsg}`);
       return false;
     }
 
     if (editData.error) {
-      console.log(`Error: ${editData.error.display}`);
+      debugLogger.log(`Error: ${editData.error.display}`);
       return false;
     }
 
@@ -769,13 +816,13 @@ export class SmartEditTool
   extends BaseDeclarativeTool<EditToolParams, ToolResult>
   implements ModifiableDeclarativeTool<EditToolParams>
 {
-  static readonly Name = 'replace';
+  static readonly Name = EDIT_TOOL_NAME;
 
   constructor(private readonly config: Config) {
     super(
       SmartEditTool.Name,
       'Edit',
-      `Replaces text within a file. Replaces a single occurrence. This tool requires providing significant context around the change to ensure precise targeting. Always use the ${ReadFileTool.Name} tool to examine the file's current content before attempting a text replacement.
+      `Replaces text within a file. Replaces a single occurrence. This tool requires providing significant context around the change to ensure precise targeting. Always use the ${READ_FILE_TOOL_NAME} tool to examine the file's current content before attempting a text replacement.
       
       The user has the ability to modify the \`new_string\` content. If modified, this will be stated in the response.
       
@@ -831,58 +878,6 @@ A good instruction should concisely answer:
     );
   }
 
-  /**
-   * Quickly checks if the file path can be resolved directly against the workspace root.
-   * @param filePath The relative file path to check.
-   * @returns The absolute path if the file exists, otherwise null.
-   */
-  private findDirectPath(filePath: string): string | null {
-    const directPath = path.join(this.config.getTargetDir(), filePath);
-    return fs.existsSync(directPath) ? directPath : null;
-  }
-
-  /**
-   * Searches for a file across all configured workspace directories.
-   * @param filePath The file path (can be partial) to search for.
-   * @returns A list of absolute paths for all matching files found.
-   */
-  private findAmbiguousPaths(filePath: string): string[] {
-    const workspaceContext = this.config.getWorkspaceContext();
-    const fileSystem = this.config.getFileSystemService();
-    const searchPaths = workspaceContext.getDirectories();
-    return fileSystem.findFiles(filePath, searchPaths);
-  }
-
-  /**
-   * Attempts to correct a relative file path to an absolute path.
-   * This function modifies `params.file_path` in place if successful.
-   * @param params The tool parameters containing the file_path to correct.
-   * @returns An error message string if correction fails, otherwise null.
-   */
-  private correctPath(params: EditToolParams): string | null {
-    const directPath = this.findDirectPath(params.file_path);
-    if (directPath) {
-      params.file_path = directPath;
-      return null;
-    }
-
-    const foundFiles = this.findAmbiguousPaths(params.file_path);
-
-    if (foundFiles.length === 0) {
-      return `File not found for '${params.file_path}' and path is not absolute.`;
-    }
-
-    if (foundFiles.length > 1) {
-      return (
-        `The file path '${params.file_path}' is too ambiguous and matches multiple files. ` +
-        `Please provide a more specific path. Matches: ${foundFiles.join(', ')}`
-      );
-    }
-
-    params.file_path = foundFiles[0];
-    return null;
-  }
-
   /**
    * Validates the parameters for the Edit tool
    * @param params Parameters to validate
@@ -895,11 +890,17 @@ A good instruction should concisely answer:
       return "The 'file_path' parameter must be non-empty.";
     }
 
-    if (!path.isAbsolute(params.file_path)) {
+    let filePath = params.file_path;
+    if (!path.isAbsolute(filePath)) {
       // Attempt to auto-correct to an absolute path
-      const error = this.correctPath(params);
-      if (error) return error;
+      const result = correctPath(filePath, this.config);
+      if (result.success) {
+        filePath = result.correctedPath;
+      } else {
+        return result.error;
+      }
     }
+    params.file_path = filePath;
 
     const workspaceContext = this.config.getWorkspaceContext();
     if (!workspaceContext.isPathWithinWorkspace(params.file_path)) {
