@@ -76,13 +76,9 @@ export abstract class BaseToolInvocation<
   constructor(
     readonly params: TParams,
     protected readonly messageBus?: MessageBus,
-  ) {
-    if (this.messageBus) {
-      console.log(
-        `[DEBUG] Tool ${this.constructor.name} created with messageBus: YES`,
-      );
-    }
-  }
+    readonly _toolName?: string,
+    readonly _toolDisplayName?: string,
+  ) {}
 
   abstract getDescription(): string;
 
@@ -90,117 +86,128 @@ export abstract class BaseToolInvocation<
     return [];
   }
 
-  shouldConfirmExecute(
-    _abortSignal: AbortSignal,
-  ): Promise<ToolCallConfirmationDetails | false> {
-    // If message bus is available, use it for confirmation
-    if (this.messageBus) {
-      console.log(
-        `[DEBUG] Using message bus for tool confirmation: ${this.constructor.name}`,
-      );
-      return this.handleMessageBusConfirmation(_abortSignal);
-    }
-
-    // Fall back to existing confirmation flow
-    return Promise.resolve(false);
-  }
-
-  /**
-   * Handle tool confirmation using the message bus.
-   * This method publishes a confirmation request and waits for the response.
-   */
-  protected async handleMessageBusConfirmation(
+  async shouldConfirmExecute(
     abortSignal: AbortSignal,
   ): Promise<ToolCallConfirmationDetails | false> {
+    if (this.messageBus) {
+      const decision = await this.getMessageBusDecision(abortSignal);
+      if (decision === 'ALLOW') {
+        return false;
+      }
+
+      if (decision === 'DENY') {
+        throw new Error(
+          `Tool execution for "${
+            this._toolDisplayName || this._toolName
+          }" denied by policy.`,
+        );
+      }
+
+      if (decision === 'ASK_USER') {
+        const confirmationDetails: ToolCallConfirmationDetails = {
+          type: 'info',
+          title: `Confirm: ${this._toolDisplayName || this._toolName}`,
+          prompt: this.getDescription(),
+          onConfirm: async (outcome: ToolConfirmationOutcome) => {
+            if (outcome === ToolConfirmationOutcome.ProceedAlways) {
+              if (this.messageBus && this._toolName) {
+                this.messageBus.publish({
+                  type: MessageBusType.UPDATE_POLICY,
+                  toolName: this._toolName,
+                });
+              }
+            }
+          },
+        };
+        return confirmationDetails;
+      }
+    }
+    return false;
+  }
+
+  protected getMessageBusDecision(
+    abortSignal: AbortSignal,
+  ): Promise<'ALLOW' | 'DENY' | 'ASK_USER'> {
     if (!this.messageBus) {
-      return false;
+      // If there's no message bus, we can't make a decision, so we allow.
+      // The legacy confirmation flow will still apply if the tool needs it.
+      return Promise.resolve('ALLOW');
     }
 
     const correlationId = randomUUID();
     const toolCall = {
-      name: this.constructor.name,
+      name: this._toolName || this.constructor.name,
       args: this.params as Record<string, unknown>,
     };
 
-    return new Promise<ToolCallConfirmationDetails | false>(
-      (resolve, reject) => {
-        if (!this.messageBus) {
-          resolve(false);
-          return;
+    return new Promise<'ALLOW' | 'DENY' | 'ASK_USER'>((resolve) => {
+      if (!this.messageBus) {
+        resolve('ALLOW');
+        return;
+      }
+
+      let timeoutId: NodeJS.Timeout | undefined;
+
+      const cleanup = () => {
+        if (timeoutId) {
+          clearTimeout(timeoutId);
+          timeoutId = undefined;
         }
-
-        let timeoutId: NodeJS.Timeout | undefined;
-
-        // Centralized cleanup function
-        const cleanup = () => {
-          if (timeoutId) {
-            clearTimeout(timeoutId);
-            timeoutId = undefined;
-          }
-          abortSignal.removeEventListener('abort', abortHandler);
-          this.messageBus?.unsubscribe(
-            MessageBusType.TOOL_CONFIRMATION_RESPONSE,
-            responseHandler,
-          );
-        };
-
-        // Set up abort handler
-        const abortHandler = () => {
-          cleanup();
-          reject(new Error('Tool confirmation aborted'));
-        };
-
-        // Check if already aborted
-        if (abortSignal.aborted) {
-          reject(new Error('Tool confirmation aborted'));
-          return;
-        }
-
-        // Set up response handler
-        const responseHandler = (response: ToolConfirmationResponse) => {
-          if (response.correlationId === correlationId) {
-            cleanup();
-
-            if (response.confirmed) {
-              // Tool was confirmed, return false to indicate no further confirmation needed
-              resolve(false);
-            } else {
-              // Tool was denied, reject to prevent execution
-              reject(new Error('Tool execution denied by policy'));
-            }
-          }
-        };
-
-        // Add event listener for abort signal
-        abortSignal.addEventListener('abort', abortHandler);
-
-        // Set up timeout
-        timeoutId = setTimeout(() => {
-          cleanup();
-          resolve(false);
-        }, 30000); // 30 second timeout
-
-        // Subscribe to response
-        this.messageBus.subscribe(
+        abortSignal.removeEventListener('abort', abortHandler);
+        this.messageBus?.unsubscribe(
           MessageBusType.TOOL_CONFIRMATION_RESPONSE,
           responseHandler,
         );
+      };
 
-        // Publish confirmation request
-        const request: ToolConfirmationRequest = {
-          type: MessageBusType.TOOL_CONFIRMATION_REQUEST,
-          toolCall,
-          correlationId,
-        };
+      const abortHandler = () => {
+        cleanup();
+        resolve('DENY');
+      };
 
-        try {
-          this.messageBus.publish(request);
-        } catch (_error) {
+      if (abortSignal.aborted) {
+        resolve('DENY');
+        return;
+      }
+
+      const responseHandler = (response: ToolConfirmationResponse) => {
+        if (response.correlationId === correlationId) {
           cleanup();
-          resolve(false);
+          if (response.requiresUserConfirmation) {
+            resolve('ASK_USER');
+          } else if (response.confirmed) {
+            resolve('ALLOW');
+          } else {
+            resolve('DENY');
+          }
         }
-      },
-    );
+      };
+
+      abortSignal.addEventListener('abort', abortHandler);
+
+      timeoutId = setTimeout(() => {
+        cleanup();
+        resolve('ASK_USER'); // Default to ASK_USER on timeout
+      }, 30000);
+
+      this.messageBus.subscribe(
+        MessageBusType.TOOL_CONFIRMATION_RESPONSE,
+        responseHandler,
+      );
+
+      const request: ToolConfirmationRequest = {
+        type: MessageBusType.TOOL_CONFIRMATION_REQUEST,
+        toolCall,
+        correlationId,
+      };
+
+      try {
+        this.messageBus.publish(request);
+      } catch (_error) {
+        cleanup();
+        resolve('ALLOW');
+      }
+    });
   }
 
   abstract execute(
@@ -283,6 +290,7 @@ export abstract class DeclarativeTool<
     readonly isOutputMarkdown: boolean = true,
     readonly canUpdateOutput: boolean = false,
     readonly messageBus?: MessageBus,
+    readonly extensionId?: string,
   ) {}
 
   get schema(): FunctionDeclaration {
@@ -406,7 +414,12 @@ export abstract class BaseDeclarativeTool<
     if (validationError) {
       throw new Error(validationError);
     }
-    return this.createInvocation(params, this.messageBus);
+    return this.createInvocation(
+      params,
+      this.messageBus,
+      this.name,
+      this.displayName,
+    );
   }
 
   override validateToolParams(params: TParams): string | null {
@@ -429,6 +442,8 @@ export abstract class BaseDeclarativeTool<
   protected abstract createInvocation(
     params: TParams,
     messageBus?: MessageBus,
+    _toolName?: string,
+    _toolDisplayName?: string,
   ): ToolInvocation<TParams, TResult>;
 }
 
@@ -562,7 +577,18 @@ export function hasCycleInSchema(schema: object): boolean {
   return traverse(schema, new Set<string>(), new Set<string>());
 }
 
-export type ToolResultDisplay = string | FileDiff | AnsiOutput;
+export interface TodoList {
+  todos: Todo[];
+}
+
+export type ToolResultDisplay = string | FileDiff | AnsiOutput | TodoList;
+
+export type TodoStatus = 'pending' | 'in_progress' | 'completed' | 'cancelled';
+
+export interface Todo {
+  description: string;
+  status: TodoStatus;
+}
 
 export interface FileDiff {
   fileDiff: string;
