@@ -6,6 +6,7 @@
 
 import { getErrorMessage, isNodeError } from './errors.js';
 import { URL } from 'node:url';
+import { ProxyAgent, setGlobalDispatcher } from 'undici';
 
 const PRIVATE_IP_RANGES = [
   /^10\./,
@@ -55,3 +56,11 @@ export async function fetchWithTimeout(
     clearTimeout(timeoutId);
   }
 }
+
+export function setGlobalProxy(proxy: string) {
+  try {
+    setGlobalDispatcher(new ProxyAgent(proxy));
+  } catch (e) {
+    console.error(`Failed to set proxy: ${getErrorMessage(e)}`);
+  }
+}
