@@ -234,4 +234,40 @@ src/*.tmp
       );
     });
   });
+  describe('Escaped Characters', () => {
+    beforeEach(async () => {
+      await setupGitRepo();
+    });
+
+    it('should correctly handle escaped characters in .gitignore', async () => {
+      await createTestFile('.gitignore', '\\#foo\n\\!bar');
+      // Create files with special characters in names
+      await createTestFile('bla/#foo', 'content');
+      await createTestFile('bla/!bar', 'content');
+
+      // These should be ignored based on the escaped patterns
+      expect(parser.isIgnored('bla/#foo')).toBe(true);
+      expect(parser.isIgnored('bla/!bar')).toBe(true);
+    });
+  });
+
+  describe('Trailing Spaces', () => {
+    beforeEach(async () => {
+      await setupGitRepo();
+    });
+
+    it('should correctly handle significant trailing spaces', async () => {
+      await createTestFile('.gitignore', 'foo\\ \nbar ');
+      await createTestFile('foo ', 'content');
+      await createTestFile('bar', 'content');
+      await createTestFile('bar ', 'content');
+
+      // 'foo\ ' should match 'foo '
+      expect(parser.isIgnored('foo ')).toBe(true);
+
+      // 'bar ' should be trimmed to 'bar'
+      expect(parser.isIgnored('bar')).toBe(true);
+      expect(parser.isIgnored('bar ')).toBe(false);
+    });
+  });
 });
