@@ -9,7 +9,18 @@ import * as addFormats from 'ajv-formats';
 // Ajv's ESM/CJS interop: use 'any' for compatibility as recommended by Ajv docs
 // eslint-disable-next-line @typescript-eslint/no-explicit-any
 const AjvClass = (AjvPkg as any).default || AjvPkg;
-const ajValidator = new AjvClass();
+const ajValidator = new AjvClass(
+  // See: https://ajv.js.org/options.html#strict-mode-options
+  {
+    // strictSchema defaults to true and prevents use of JSON schemas that
+    // include unrecognized keywords. The JSON schema spec specifically allows
+    // for the use of non-standard keywords and the spec-compliant behavior
+    // is to ignore those keywords. Note that setting this to false also
+    // allows use of non-standard or custom formats (the unknown format value
+    // will be logged but the schema will still be considered valid).
+    strictSchema: false,
+  },
+);
 // eslint-disable-next-line @typescript-eslint/no-explicit-any
 const addFormatsFunc = (addFormats as any).default || addFormats;
 addFormatsFunc(ajValidator);
