@@ -43,10 +43,17 @@ async function main() {
     outfile: 'dist/extension.cjs',
     external: ['vscode'],
     logLevel: 'silent',
+    banner: {
+      js: `const import_meta = { url: require('url').pathToFileURL(__filename).href };`,
+    },
+    define: {
+      'import.meta.url': 'import_meta.url',
+    },
     plugins: [
       /* add to the end of plugins array */
       esbuildProblemMatcherPlugin,
     ],
+    loader: { '.node': 'file' },
   });
   if (watch) {
     await ctx.watch();
